#   Simple Backup - Configuration GUI (GTK+)
#
#   Copyright (c)2009-2010: Jean-Peer Lorenz <peer.loz@gmx.net>
#   Copyright (c)2007-2009: Ouattara Oumar Aziz <wattazoum@gmail.com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

from gettext import gettext as _

import re
import os
import sys
import types

import gobject
import glib
import gtk

from sbackup.util import log
from sbackup.util import system
from sbackup.util.log import LogFactory
from sbackup.util.exceptions import SBException

from sbackup.core import ConfigManager
from sbackup.core.ConfigManager import ConfigurationFileHandler
from sbackup.core.ConfigManager import ConfigManagerStaticData

import sbackup.util as Util

from sbackup.ui.GladeGnomeApp import GladeGnomeApp
from sbackup.ui import misc
from sbackup.ui import gtk_rsrc

from sbackup.fs_backend import fam

from sbackup.util import exceptions
from sbackup.util import pathparse
from sbackup.util import constants
from sbackup.util import local_file_utils


sys.excepthook = misc.except_hook
system.launch_dbus_if_required()


class SBconfigGTK(GladeGnomeApp):
    """
    
    @todo: Unify displaying of error messages/dialog boxes!
    @todo: Strictly separate UI from core. Don't set defaults from the UI (business logic in handler).
    @todo: Use functions from ConfigManager to set the paths in
            a consistent manner.         
    @todo: Configuration handling must be reviewed. Direct manipulation
            of the configuration from widget's signal handler is *really*
            errorprone and hard to debug (e.g. clearing a text input field
            from the source code/application side before filling it with new
            content currently yields in the removal of the according
            config option and the config is unintentionally changed). 
    """
    # why class variables?
    configman = None
    conffile = None
    orig_configman = None
    plugin_manager = None

    def __init__(self):
        """Default constructor.
        """
        # it is distinguished between the 'current' conffile and
        # the 'default file' configuring the default profile
        self.default_conffile = None
        self.__configFileHandler = ConfigurationFileHandler()
        _path_conffile = self.__configFileHandler.get_default_conffile_fullpath()
        if os.path.exists(_path_conffile):
            self.default_conffile = _path_conffile
            self.conffile = self.default_conffile
            self.configman = ConfigManager.ConfigManager(self.default_conffile)

            # hack to get rid of schedule settings in non-admin profiles
            # we just remove existing schedules from the config files
            # and don't allow new settings by disabling the schedule page
            if os.geteuid() != 0:
                self.configman.remove_schedule()
                self.configman.saveConf()
            # end of hack

            self.orig_configman = ConfigManager.ConfigManager(self.default_conffile)
        else:
            self.configman = ConfigManager.ConfigManager()
            self.orig_configman = None
            gobject.idle_add(_notify_new_default_profile_created)

        self.logger = LogFactory.getLogger()
        self.__destination_uri_obj = None
        self.__destination_hdl = None
        self.__destination_failure = False

        GladeGnomeApp.__init__(self,
                               app_name = "SBackup",
                               app_version = "0.2",
                               filename = Util.get_resource_file(gtk_rsrc.CONFIGGUI_GLADEFILE),
                               top_window = gtk_rsrc.CONFIGGUI_TOPWINDOW,
                               widget_list = gtk_rsrc.get_configgui_widgets(),
                               handlers = gtk_rsrc.get_configgui_handlers())

        gtk.window_set_default_icon_from_file(Util.get_resource_file(constants.CONFIG_ICON_FILENAME))

        # hide the schedule tab if not root
        if not system.is_superuser():
            self.__enable_schedule_page(enable = False)
            self.widgets['label_schedule_page'].set_tooltip_text(\
            _('Scheduled backups are available for Administrator users only.'))

        # Initiate all data structures
        # Paths to be included or excluded
        self.include = gtk.ListStore(str)
        self.includetv = self.widgets["includetv"]
        self.includetv.set_model(self.include)
        cell = gtk.CellRendererText()
        cell.set_property('editable', True)
        cell.connect('edited', self.cell_edited_callback, (self.include, "dirconfig", 1))
        column = gtk.TreeViewColumn(_('Name'), cell, text = 0)
        self.includetv.append_column(column)

        self.ex_paths = gtk.ListStore(str)
        self.ex_pathstv = self.widgets["ex_pathstv"]
        self.ex_pathstv.set_model(self.ex_paths)
        cell1 = gtk.CellRendererText()
        cell1.set_property('editable', True)
        cell1.connect('edited', self.cell_edited_callback, (self.ex_paths, "dirconfig", 0))
        column1 = gtk.TreeViewColumn(_('Name'), cell1, text = 0)
        self.ex_pathstv.append_column(column1)

        # Excluded file types and general regular expressions
        self.ex_ftype = gtk.ListStore(str, str)
        self.ex_ftypetv = self.widgets["ex_ftypetv"]
        self.ex_ftypetv.set_model(self.ex_ftype)
        cell3 = gtk.CellRendererText()
        column3 = gtk.TreeViewColumn(_('File Type'), cell3, text = 0)
        cell2 = gtk.CellRendererText()
        column2 = gtk.TreeViewColumn('Ext.', cell2, text = 1)
        self.ex_ftypetv.append_column(column3)
        self.ex_ftypetv.append_column(column2)

        self.ex_regex = gtk.ListStore(str)
        self.ex_regextv = self.widgets["ex_regextv"]
        self.ex_regextv.set_model(self.ex_regex)
        cell4 = gtk.CellRendererText()
        cell4.set_property('editable', True)
        cell4.connect('edited', self.cell_regex_edited_callback)
        column4 = gtk.TreeViewColumn('Name', cell4, text = 0)
        self.ex_regextv.append_column(column4)

        # Profile Manager
        # [ enable , profilename, cfPath ]
        self.profiles = gtk.ListStore(bool, str, str)
        # add the default profile and disable any modification to it
        self.profiles.append([True,
            ConfigManagerStaticData.get_default_profilename(),
            self.__configFileHandler.get_default_conffile_fullpath()])
        for i, v in self.configman.getProfiles().iteritems() :
            self.profiles.append([v[1], i, v[0]])
        self.profilestv = self.widgets['profilesListTreeView']
        self.profilestv.set_model(self.profiles)

        cell8, cell9 = gtk.CellRendererToggle(), gtk.CellRendererText()
        cell8.set_active(True)
        cell8.connect("toggled", self.on_prfEnableCB_toggled)

        enableCBColumn = gtk.TreeViewColumn(_("Enable"), cell8, active = 0)
        prfNameColumn = gtk.TreeViewColumn(_("Profile Name"), cell9, text = 1)

        self.profilestv.append_column(enableCBColumn)
        self.profilestv.append_column(prfNameColumn)

        # The split size coices        
        self.splitSizeLS = misc.set_model(widget = self.widgets['splitsizeCB'],
                                          values = ConfigManagerStaticData.get_splitsize_dict())

        self.__model_remote_services = misc.set_model(widget = self.widgets['cmb_set_remote_service'],
                                                      values = fam.get_remote_services_avail())

        self._fill_widgets_from_config(probe_fs = True)
        
        self.xml.signal_autoconnect(self.cb_dict)
        

    def isConfigChanged(self, force_the_change = False):
        """Checks whether the current configuration has changed compared to
        the configuration which was originally loaded resp. stored on last
        save action. The result (irrespective whether it was forced or not)
        is returned by the method.
        
        @param force_the_change: Flag that that forces the check to be True
                                (i.e. the method acts as there were changes
                                regardless of the real test result)
        
        @return: True if the config has changed, False otherwise
        @rtype: Boolean                        
        """
        changed = not self.configman.isConfigEquals(self.orig_configman)
        if force_the_change == True:
            changed = True
        self.widgets['save'].set_sensitive(changed)
        self.widgets['saveButton'].set_sensitive(changed)
        return changed

    def ask_save_config(self):
        """Checks whether the configuration has changed and displays
        a dialog window if so. The user then can decide to save, not to save
        the configuration or to cancel the process.
        
        @return: False if the user pressed 'yes' or 'no'; True in any other case
        @rtype: Boolean
        """
        cancelled = False
        changed = self.isConfigChanged()
        if changed:
            question = self.widgets['askSaveDialog']
            question.set_title("")
            response = question.run()
            question.hide()
            if response == gtk.RESPONSE_YES:
                self.on_save_clicked()
            elif response == gtk.RESPONSE_NO:
                pass
            else:
                cancelled = True
        return cancelled

    def __fill_dir_widgets_from_config(self):
        """Fills the directory include and exclude tabs according to
        the values found in the current configuration.
        """
        _section = "dirconfig"
        self.include.clear()
        self.ex_paths.clear()
        dirconfig = self.configman.get_dirconfig_local()
        if dirconfig is not None:
            for _item, _value in dirconfig:
                if _value == 1:
                    self.include.append([_item])
                elif _value == 0:
                    self.ex_paths.append([_item])

    def __fill_regex_widgets_from_config(self):
        # regexp excludes
        _known_ftypes_dict = ConfigManagerStaticData.get_known_ftypes_dict()
        _invalid_regex_found = False
        _invalid_regex = ""
        self.ex_ftype.clear()
        self.ex_regex.clear()
        if self.configman.has_option("exclude", "regex") :
            r = self.configman.get("exclude", "regex")
            if not Util.is_empty_regexp(r):
                _list = str(r).split(",")
                for i in _list:
# Bugfix LP #258542 
                    if re.match(r"\\\.\w+\$", i):
                        _ftype = i[2:len(i) - 1]
                        if _ftype in _known_ftypes_dict:
                            self.ex_ftype.append([_known_ftypes_dict[_ftype], _ftype])

                        else:
                            self.ex_ftype.append([_("Custom"), _ftype])
                    else:
                        if (not Util.is_empty_regexp(i)) and Util.is_valid_regexp(i):
                            self.ex_regex.append([i])
                        else:
                            r = Util.remove_conf_entry(r, i)
                            self.logger.warning(_("Invalid or empty regular expression ('%s') found in configuration. Removed.") % i)
                            _invalid_regex_found = True
                            _invalid_regex = "%s, %s" % (_invalid_regex, i)

        if _invalid_regex_found:
            self.configman.set("exclude", "regex", r)
            self.isConfigChanged()
            _msg = _("Invalid or empty regular expressions found\nin configuration file:\n'%s'\n\nThese expressions are not used and were\nremoved from the configuration.")\
                % (_invalid_regex.lstrip(","))
            gobject.idle_add(misc.show_errdialog, _msg,
                             self.__get_application_widget())

    def __fill_max_filesize_widgets_from_config(self):
        """Sets the UI elements for 'maximum size limit' to the value
        specified in configuration.
        """
        if self.configman.has_option("exclude", "maxsize"):
            _maxsize = self.configman.getint("exclude", "maxsize")
            self.widgets["ex_maxsize"].set_value(_maxsize / (1024 * 1024))
            if _maxsize > 0:
                self.widgets["ex_maxsize"].set_sensitive(True)
                self.widgets["ex_max"].set_active(True)
            else:
                self.widgets["ex_maxsize"].set_sensitive(False)
                self.widgets["ex_max"].set_active(False)
        else:
            self.widgets["ex_maxsize"].set_sensitive(False)
            self.widgets["ex_max"].set_active(False)

    def __fill_followlinks_widgets_from_config(self):
        """Sets the UI elements for 'followlinks' to the value
        specified in configuration.
        """
        section = "general"
        option = "followlinks"
        followlinks = False
        if self.configman.has_option(section, option):
            config_fl = self.configman.get(section, option)
            if config_fl is True or config_fl == 1 or config_fl == "1":
                followlinks = True
        self.widgets["followlinks"].set_active(followlinks)

    def __fill_compression_widgets_from_config(self):
        """Sets the UI elements for 'compression format' to the value
        specified in configuration.
        """
        cformats = ConfigManagerStaticData.get_compr_formats()
        cformatOpt = self.configman.get_compress_format()
        if cformatOpt not in cformats:
            cformatOpt = 'none'
        cformatIndex = cformats.index(cformatOpt)
        self.widgets["cformat"].set_active(cformatIndex)

    def __fill_purge_widgets_from_config(self):
        """Sets the UI elements for 'purge' to the value
        specified in configuration.
        """
        if self.configman.has_option("general", "purge"):
            self.logger.debug("Setting purge")
            if self.configman.get("general", "purge") == "log":
                self.widgets['logpurgeradiobutton'].set_active(True)
                self.widgets["purgecheckbox"].set_active(True)
            else:
                try :
                    purge = int(self.configman.get("general", "purge"))
                except Exception, e:
                    self.logger.error("Purge value '%s' is invalid: '%s'" \
                                % (self.configman.get("general", "purge"), e))
                    purge = 30
                self.widgets['purgedays'].set_text(str(purge))
                self.widgets['purgeradiobutton'].set_active(True)
                self.widgets["purgedays"].set_sensitive(True)
                self.on_purgedays_changed()
                self.widgets['purgecheckbox'].set_active(True)
        else:
            self.widgets["purgecheckbox"].set_active(False)
        self.on_purgecheckbox_toggled()

    def __fill_max_inc_widgets_from_config(self):
        """Sets the UI elements for 'Maximum of inc' to the value
        specified in configuration.
        """
        if self.configman.has_option("general", "maxincrement"):
            self.widgets["time_maxinc"].set_value(\
                            int(self.configman.get("general", "maxincrement")))

    def __fill_log_widgets_from_config(self):
        """Sets the UI elements for 'log' to the value
        specified in configuration.
        """
        _default_config = ConfigManager.get_default_config_obj()

        if not self.configman.has_section("log"):
            self.configman.add_section("log")

        if not self.configman.has_option("log", "level"):
            self.configman.set("log", "level", _default_config.get_loglevel())

        if not self.configman.has_option("log", "file"):
            self.configman.set_logdir(_default_config.get_logdir())
            self.configman.set_logfile_templ_to_config()

        loglevel = str(self.configman.get("log", "level"))  # LP #1159705
        valid_levels = ConfigManagerStaticData.get_valid_loglevels()
        selection = valid_levels[loglevel][1]
        self.widgets["loglevelcombobox"].set_active(selection)
        self.widgets["logfilechooser"].set_current_folder(self.configman.get_logdir())

    def __fill_report_widgets_from_config(self):
        """Sets the UI elements for 'report' to the value
        specified in configuration.

        @todo: Handling of non-existing settings must be removed and unified.
        """
        _from = ""
        _to = ""
        _server = ""
        _port = ""
        _user = ""
        _passw = ""
        _cert = None
        _key = None

        if self.configman.has_section("report"):
            if self.configman.has_option("report", "from"):
                _from = self.configman.get("report", "from")

            if self.configman.has_option("report", "to"):
                _to = self.configman.get("report", "to")

            if self.configman.has_option("report", "smtpserver"):
                _server = self.configman.get("report", "smtpserver")

            if self.configman.has_option("report", "smtpport"):
                _port = self.configman.get("report", "smtpport")

            if self.configman.has_option("report", "smtpuser") or \
               self.configman.has_option("report", "smtppassword"):
                self.widgets["smtplogincheckbox"].set_active(True)
                self.widgets['smtplogininfo'].set_sensitive(True)

                if self.configman.has_option("report", "smtpuser"):
                    _user = self.configman.get("report", "smtpuser")

                if self.configman.has_option("report", "smtppassword"):
                    _passw = self.configman.get("report", "smtppassword")
            else:
                self.widgets["smtplogincheckbox"].set_active(False)
                self.widgets['smtplogininfo'].set_sensitive(False)

            if self.configman.has_option("report", "smtptls"):
                self.widgets["TLScheckbutton"].set_active(True)
                self.widgets['TLSinfos'].set_sensitive(True)
            else:
                self.widgets["TLScheckbutton"].set_active(False)

            if self.configman.has_option("report", "smtpcert") or \
               self.configman.has_option("report", "smtpkey"):
                self.widgets["SSLradiobutton"].set_active(True)
                self.__enable_ssl_options(enable = True)

                if self.configman.has_option("report", "smtpcert"):
                    _cert = self.configman.get("report", "smtpcert")
                    self.widgets['crtfilechooser'].set_filename(_cert)

                if self.configman.has_option("report", "smtpkey"):
                    _key = self.configman.get("report", "smtpkey")
                    self.widgets['keyfilechooser'].set_filename(_key)
            else :
                self.widgets["TLSradiobutton"].set_active(True)
                self.__enable_ssl_options(enable = False)

        self.widgets["smtpfrom"].set_text(_from)
        self.widgets["smtpto"].set_text(_to)
        self.widgets["smtpserver"].set_text(_server)
        self.widgets["smtpport"].set_text(_port)
        self.widgets["smtplogin"].set_text(_user)
        self.widgets["smtppassword"].set_text(_passw)

    def __fill_schedule_widgets(self, from_func):
        """Sets the UI elements for 'schedule' to the value
        specified in configuration (only from configuration or from file
        and file system).
        
        @param from_func: function object reference which is called in order
                            to retrieve the current cron state
                            
        @note: Purpose of the given function object is to use this method
                with `ConfigManager.get_schedule` as well as
                `ConfigManager.get_schedule_and_probe`.
                
        @todo: Implement signature checking. 
        """
        if not isinstance(from_func, types.MethodType):
            raise TypeError("Given parameter 'from_func' must be of type "\
                            "Method. Got %s instead." % (type(from_func)))

        if system.is_superuser() and self.configman.is_default_profile():
            self.__enable_schedule_page(enable = True)

            croninfos = from_func()    # = (isCron, val)

            # any schedule information was found
            if  croninfos is not None:
                # scheduled using Cron i.e. a custom setting
                if croninfos[0] == 1:
                    self.__set_schedule_option('custom')
                    self.__set_value_txtfld_custom_cronline(croninfos[1])

                # scheduled using Anacron
                elif croninfos[0] == 0:
                    self.__set_schedule_option('simple')
                    self.__set_value_cmbbx_simple_schedule_freq(croninfos[1])

            # no scheduled backups
            else:
                self.__set_schedule_option('no')
        else:
            self.__enable_schedule_page(enable = False)

    def __fill_splitsize_widgets_from_config(self):
        if self.configman.has_option("general", "splitsize") :
            model = self.widgets["splitsizeCB"].get_model()
            custom = True
            for i in range(0, len(model)) :
                if model[i][1] == int(self.configman.get("general",
                                                        "splitsize")) / 1024:
                    self.widgets["splitsizeCB"].set_active(i)
                    self.__enable_splitsize_custom_option(enable = False)
                    custom = False
            if custom:
                # NOTE: if we don't do this is this order, the handler
                #        on splitsizeCB will overide splitsizeSB value
                self.widgets["splitsizeSB"].set_value(\
                        int(self.configman.get("general", "splitsize")) / 1024)
                self.widgets["splitsizeCB"].set_active(0)

    def __dest_from_config_helper(self):
        """Creates and returns a TargetHandlerInstance with set
        destination path as stored/set in ConfigurationManager.
        """
        if self.__destination_uri_obj is None:
            self.__destination_uri_obj = pathparse.UriParser()
            
        self.__destination_uri_obj.set_and_parse_uri(uri = self.configman.get_destination_path())
        _dest = fam.get_fam_target_handler_facade_instance()
        _dest.set_destination(self.__destination_uri_obj.uri)
        return _dest

    def __fill_destination_widgets(self):
        """Helper method which fills the UI widgets related to
        the backup target (i.e. destination).
        """
        _dest = self.__dest_from_config_helper()
        ctarget = _dest.query_dest_display_name()
        self.logger.debug("Current destination: %s" % ctarget)

        if _dest.is_local():
            self.__set_target_label("local", ctarget)
            if not _dest.dest_path_exists():
                _sec = _("A common mistake is a not mounted external disk.")
                _msg = _("Backup destination folder `%s` does not exist.\n\nPlease make "\
                  "sure the missing directory exists and check your settings on the "\
                  "destination settings page.") % ctarget
                _hdl = _("Unable to open backup destination")
                gobject.idle_add(misc.show_errdialog, _msg,
                                  self.__get_application_widget(),
                                  _hdl, _sec)
        else:
            self.__set_target_label("remote", ctarget)

    def _fill_widgets_from_config(self, probe_fs):
        """Prefill the GTK window with config infos.
        
        @param probe_fs: Flag whether to probe the filesystem for schedule info
        
        @todo: Opening of directories (e.g. target) must be unified over all
               modules that use such functionality!
        """
        if not isinstance(probe_fs, types.BooleanType):
            raise TypeError("Given parameter must be of boolean type. "\
                            "Got %s instead." % (type(probe_fs)))
        try:
            # General
            self.__fill_max_inc_widgets_from_config()
            self.__fill_compression_widgets_from_config()
            self.__fill_splitsize_widgets_from_config()

            # dirconfig and excludes
            self.__fill_dir_widgets_from_config()
            self.__fill_regex_widgets_from_config()

            # other exclude reasons
            self.__fill_max_filesize_widgets_from_config()
            self.__fill_followlinks_widgets_from_config()

            # target (= destination)
            self.__fill_destination_widgets()

            # schedule - with probing the filesystem or without
            if probe_fs:
                sched_func = self.configman.get_schedule_and_probe
            else:
                sched_func = self.configman.get_schedule
            self.__fill_schedule_widgets(from_func = sched_func)

            # Purging
            self.__fill_purge_widgets_from_config()

            # Log and report
            self.__fill_log_widgets_from_config()
            self.__fill_report_widgets_from_config()

            self.__fill_statusbar_from_config()
        except exceptions.NonValidOptionException, error:
            gobject.idle_add(self.__config_invalid_cb, error, probe_fs)
        else:
#            self.__set_default_focus()
            self.isConfigChanged()

    def __config_invalid_cb(self, error, probe_fs):
        dialog = gtk.MessageDialog(type = gtk.MESSAGE_ERROR,
                                   flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                   buttons = gtk.BUTTONS_CLOSE,
                                   message_format = _("%s\n\n"\
                                                      "A backup profile using default values was created. "\
                                                      "Save the new configuration in order to use it or "\
                                                      "check your existing configuration file manually.\n\n"\
                                                      "Important note: Saving will overwrite your "\
                                                      "existing invalid configuration.") % str(error))
        dialog.run()
        dialog.destroy()
        self.configman = ConfigManager.ConfigManager()
        self.orig_configman = None
        gobject.idle_add(self._fill_widgets_from_config, probe_fs)

#    def __set_default_focus(self):
#        self.widgets['label_general_page'].grab_focus()

    def __fill_statusbar_from_config(self):
        """Sets the profile name and the user mode.
        """
        stattxt = _("Current profile: %s") % self.configman.getProfileName()
        stattxt = misc.get_statusbar_msg_mode(stattxt)
        self.widgets['statusBar'].push(stattxt)

    def __enable_splitsize_custom_option(self, enable = True):
        """Enables resp. disables widgets for setting a custom archive
        splitsize.
        """
        self.widgets["splitsizeSB"].set_sensitive(enable)
        self.widgets["label_splitsize_custom"].set_sensitive(enable)

    def __enable_target_option(self, option):
        """The widgets within the 'Destination' page are
        enabled/disabled/set according to the given option.
        Unusable widgets are automatically disabled.
        """
        def __enable_local_target(enable = True):
            """The widgets within the 'Destination' page are
            enabled/disabled/set according to the given local target directory.
            """
            self.widgets["dest_local"].set_sensitive(enable)
            self.widgets["btn_browse_local"].set_sensitive(enable)

        def __enable_remote_target(enable = True):
            """The widgets within the 'Destination' page are
            enabled/disabled/set according to the given remote target.
            """
            self.widgets["dest_remote_light"].set_sensitive(enable)
            self.widgets["dest_remote"].set_sensitive(enable)
            self.widgets["btn_set_remote"].set_sensitive(enable)

        if option == "local":
            __enable_local_target(enable = True)
            __enable_remote_target(enable = False)
        elif option == "remote":
            __enable_local_target(enable = False)
            __enable_remote_target(enable = True)
        else:
            raise ValueError("Unknown target option given.")

    def __set_target_label(self, option, value):
        """Selects resp. sets the given choice for backup target.
        Valid options are 'local' and 'remote'.
        """
        self.__enable_target_option(option)
        twidget = None
        if option == "local":
            twidget = self.widgets["dest2"]
            label = self.widgets["dest_local"]
        elif option == "remote":
            twidget = self.widgets["dest3"]
            label = self.widgets["dest_remote"]
        else:
            raise ValueError("Unknown target option given.")
        label.set_text(value)
        twidget.set_active(True)
        twidget.grab_focus()

    def already_inc (self, configlist, toInclude):
        """configlist is like self.conf.items( "dirconfig" )
         
        @return: True if the dir is already included, False if not
        """
        for i, v in configlist :
            if v == "1" and i == toInclude :
                # the chosen item match an included one 
                dialog = gtk.MessageDialog(flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, buttons = gtk.BUTTONS_CLOSE, message_format = _("Already included item !"))
                dialog.run()
                dialog.destroy()
                return True
        # No match found
        return False

    def already_ex (self, configlist, toExclude):
        """configlist is like self.conf.items( "dirconfig" )
        
        @return: True if the dir is already excluded, False if not
        """
        for i, v in configlist :
            if v == "0" and i == toExclude :
                # the chosen item match an included one 
                dialog = gtk.MessageDialog(flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, buttons = gtk.BUTTONS_CLOSE, message_format = _("Already excluded item !"))
                dialog.run()
                dialog.destroy()
                return True
        # No match found
        return False

    def cell_regex_edited_callback(self, cell, path, new_text):
        # Check if new path is empty
        if Util.is_empty_regexp(new_text):
            misc.show_errdialog(parent = self.__get_application_widget(),
                                message_str = \
                _("Empty expression. Please enter a valid regular expression."))
        else:
            if Util.is_valid_regexp(new_text):
                # Remove old expression and add the new one
                value = self.ex_regex[path][0]
                r = self.configman.get("exclude", "regex")
                r = Util.remove_conf_entry(r, value)
                r = r + r"," + new_text.strip()
                r = r.strip(",")
                self.configman.set("exclude", "regex", r)
                self.ex_regex[path][0] = new_text
                self.isConfigChanged()
            else:
                misc.show_errdialog(parent = self.__get_application_widget(),
                                    message_str = \
                                _("Provided regular expression is not valid."))

    def cell_edited_callback(self, cell, path, new_text, data):
        # Check if new path is empty
        if (new_text == None) or (new_text == ""):
            dialog = gtk.MessageDialog(type = gtk.MESSAGE_ERROR,
                        flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                        buttons = gtk.BUTTONS_CLOSE,
                        message_format = _("Empty filename or path. Please enter a valid filename or path."))
            dialog.run()
            dialog.destroy()
            return
        # Check if new path exists and asks the user if path does not exists
        if not os.path.exists(new_text):
            dialog = gtk.MessageDialog(type = gtk.MESSAGE_QUESTION,
                        flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                        buttons = gtk.BUTTONS_YES_NO,
                        message_format = _("It seems the path you entered does not exists. Do you want to add this incorrect path?"))
            response = dialog.run()
            dialog.destroy()
            if response == gtk.RESPONSE_NO:
                return

        model, section, value = data
        self.configman.remove_option(section, model[path][0])
        model[path][0] = new_text
        self.configman.set(section, new_text, value)
        self.isConfigChanged()

    def on_ftype_toggled(self, *args): #IGNORE:W0613
        if self.widgets["ftype_st"].get_active():
            self.widgets["ftype_box"].set_sensitive(True)
            self.widgets["ftype_custom_ex"].set_sensitive(False)
        elif self.widgets["ftype_custom"].get_active():
            self.widgets["ftype_box"].set_sensitive(False)
            self.widgets["ftype_custom_ex"].set_sensitive(True)

    def on_ftype_st_box_changed(self, *args): #IGNORE:W0613
#TODO: on_ftype_st_box_changed
#        print("TODO: on_ftype_st_box_changed")
        pass

    def on_save_activate(self, *args): #IGNORE:W0613
        self.on_save_clicked()

    def on_save_as_activate(self, *args): #IGNORE:W0613
        dialog = gtk.FileChooserDialog(title = _("Save configuration as..."),
                                parent = self.__get_application_widget(),
                                action = gtk.FILE_CHOOSER_ACTION_SAVE,
                                buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                          gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)
        response = dialog.run()
        if response == gtk.RESPONSE_OK :
            self.configman.saveConf(dialog.get_filename())
        elif response == gtk.RESPONSE_CANCEL:
            pass
        dialog.destroy()

    def on_menu_help_activate(self, button):
        misc.open_uri("ghelp:sbackup")

    def on_menu_about_activate(self, *args): #IGNORE:W0613
        misc.show_about_dialog(set_transient_for = self.widgets["sbackupConfApp"])

    def on_reload_clicked(self, *args): #IGNORE:W0613
        self.configman = ConfigManager.ConfigManager(self.conffile)
        # hack to get rid of schedule settings in non-default profiles
        # we just remove existing schedules from the non-default config files
        # and don't allow new settings by disabling the schedule page
        if not self.configman.is_default_profile():
            self.configman.remove_schedule()
            self.configman.saveConf()
        # end of hack
        self.orig_configman = ConfigManager.ConfigManager(self.conffile)
        self._fill_widgets_from_config(probe_fs = True)
        self.isConfigChanged()
        self.logger.debug("Config '%s' loaded" % self.conffile)

    def on_save_clicked(self, *args): #IGNORE:W0613
        self.logger.debug("Saving Config")
        self.configman.saveConf()
        self.conffile = self.configman.conffile
        if not self.default_conffile:
            self.default_conffile = self.conffile
        self.orig_configman = ConfigManager.ConfigManager(self.configman.conffile)
        self.isConfigChanged()

    def on_backup_clicked(self, *args): #IGNORE:W0613
        cancelled = self.ask_save_config()
        if not cancelled:
            dialog = self.widgets["dialog_make_backup"]
            chkbtn_full_bak = self.widgets["checkbtn_make_backup_full"]
            btn_cancel = self.widgets['btn_cancel_make_backup']
            btn_cancel.grab_focus()

            response = dialog.run()
            _full_bak = chkbtn_full_bak.get_active()
            dialog.hide()

            if response == gtk.RESPONSE_APPLY:
                _path_to_app = Util.get_resource_file(constants.BACKUP_COMMAND)
                _cmd = [_path_to_app]

                if _full_bak is True:
                    _cmd.append("--full")

                _env = None
                if system.is_superuser():
                    _env = {}   # clear environment
                system.exec_command_async(args = _cmd, env = _env)
                misc.show_infodialog(message_str = _("A backup process is now executed in the background.\n\nYou can monitor the progress of the backup by means of the status indicator displayed in the notification area."),
                                     parent = self.top_window, headline_str = _("Backup process started"))
            elif response == gtk.RESPONSE_CANCEL or \
                 response == gtk.RESPONSE_DELETE_EVENT:
                pass
            else:
                self.logger.error(_("Unexpected dialog response: %s") % response)
                raise ValueError("Unexpected dialog response: %s" % response)

    def on_cformat_changed(self, *args): #IGNORE:W0613
        """
        handle that sets the compression format
        """
        selected = self.widgets["cformat"].get_active()
        cformats = ConfigManagerStaticData.get_compr_formats()
        if 0 <= selected < len(cformats):
            self.configman.set("general", "format", cformats[selected])
        else :
            self.configman.remove_option("general", "format")

        if selected == cformats.index("none"):
            # 'none' -> activate split functionality
            self.widgets['splitsizevbox'].set_sensitive(True)
        else :
            self.widgets["splitsizeCB"].set_active(1)
            self.widgets['splitsizevbox'].set_sensitive(False)
            if self.configman.has_option("general", "splitsize"):
                self.configman.remove_option("general", "splitsize")
            self.on_splitsizeCB_changed()
        self.isConfigChanged()

    def on_cmb_set_remote_service_changed(self, *args): #IGNORE:W0613
        pass
#        dialog = self.widgets['dialog_connect_remote']
#        cmbbx = self.widgets['cmb_set_remote_service']
#        model = cmbbx.get_model()
#        label, value = model[cmbbx.get_active()]
#        print "on_cmb_set_remote_service_changed - label: %s value: %s" % (label, value)
#        _user = self.widgets['entry_set_remote_user']
#        _user_lab = self.widgets['label_set_remote_user']
#        _pass = self.widgets['entry_set_remote_pass']
#        _pass_lab = self.widgets['label_set_remote_pass']
#
#        if value == REMOTE_SERVICE_FTP_PUBLIC:
#            _user.hide()
#            _user_lab.hide()
#            _pass.hide()
#            _pass_lab.hide()
#        else:
#            _user.show()
#            _user_lab.show()
#            _pass.show()
#            _pass_lab.show()
#        _width, _height = dialog.get_size()
#        dialog.resize(width = _width, height = 1)

    def on_splitsizeCB_changed(self, *args): #IGNORE:W0613
        """
        """
        model = self.widgets["splitsizeCB"].get_model()
        label, value = model[self.widgets["splitsizeCB"].get_active()]
        if value != -1 :
            self.__enable_splitsize_custom_option(enable = False)
            self.configman.set("general", "splitsize", value * 1024)
        else :
            # activate Spin box
            self.__enable_splitsize_custom_option(enable = True)
            val = self.widgets['splitsizeSB'].get_value_as_int()
            self.configman.set("general", "splitsize", val * 1024)
        self.isConfigChanged()

    def on_splitsizeSB_value_changed(self, *args): #IGNORE:W0613
        """
        """
        val = int(self.widgets['splitsizeSB'].get_value())
        self.configman.set("general", "splitsize", val * 1024)
        self.isConfigChanged()

    def on_inc_addfile_clicked(self, *args): #IGNORE:W0613
        self.__check_for_section("dirconfig")
        dialog = gtk.FileChooserDialog(title = _("Include file..."),
                                parent = self.__get_application_widget(),
                                action = gtk.FILE_CHOOSER_ACTION_OPEN,
                                buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                          gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)
        filter = gtk.FileFilter()
        filter.set_name(_("All files"))
        filter.add_pattern("*")
        dialog.add_filter(filter)

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            _file = _prepare_filename(dialog.get_filename())
            _enc_file = _escape_path(_file)
            if not self.already_inc(self.configman.items("dirconfig", raw = True), _enc_file):
                self.include.append([_file])
                self.configman.set("dirconfig", _enc_file, "1")
                self.isConfigChanged()
        dialog.destroy()

    def __check_for_section(self, section):
        if not self.configman.has_section(section):
            self.configman.add_section(section)

    def on_inc_adddir_clicked(self, *args): #IGNORE:W0613
        self.__check_for_section("dirconfig")
        dialog = gtk.FileChooserDialog(title = _("Include folder..."),
                                parent = self.__get_application_widget(),
                                action = gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                          gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            _dir = _prepare_dirname(dialog.get_filename())
            _enc_dir = _escape_path(_dir)
            if not self.already_inc(self.configman.items("dirconfig", raw = True), _enc_dir):
                self.include.append([_dir])
                self.configman.set("dirconfig", _enc_dir, "1")
                self.isConfigChanged()
        dialog.destroy()

    def on_inc_del_clicked(self, *args):
        self.__check_for_section("dirconfig")
        (store, iter) = self.includetv.get_selection().get_selected()
        if store and iter:
            enc_val = _escape_path(store.get_value(iter, 0))
            self.configman.remove_option("dirconfig", enc_val)
            self.isConfigChanged()
            store.remove(iter)

    def on_ex_addfile_clicked(self, *args):
        self.__check_for_section("dirconfig")
        dialog = gtk.FileChooserDialog(title = _("Exclude file..."),
                                parent = self.__get_application_widget(),
                                action = gtk.FILE_CHOOSER_ACTION_OPEN,
                                buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                         gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)
        filter = gtk.FileFilter()
        filter.set_name(_("All files"))
        filter.add_pattern("*")
        dialog.add_filter(filter)

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            _file = _prepare_filename(dialog.get_filename())
            _enc_file = _escape_path(_file)
            if not self.already_inc(self.configman.items("dirconfig", raw = True), _enc_file):
                self.ex_paths.append([_file])
                self.configman.set("dirconfig", _enc_file, "0")
                self.isConfigChanged()
        dialog.destroy()

    def on_ex_adddir_clicked(self, *args):
        self.__check_for_section("dirconfig")
        dialog = gtk.FileChooserDialog(title = _("Exclude folder..."),
                                parent = self.__get_application_widget(),
                                action = gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                         gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            _dir = _prepare_dirname(dialog.get_filename())
            _enc_dir = _escape_path(_dir)
            if not self.already_ex(self.configman.items("dirconfig", raw = True), _enc_dir):
                self.ex_paths.append([_dir])
                self.configman.set("dirconfig", _enc_dir, "0")
                self.isConfigChanged()
        dialog.destroy()

    def on_ex_delpath_clicked(self, *args):
        self.__check_for_section("dirconfig")
        (store, iter) = self.ex_pathstv.get_selection().get_selected()
        if store and iter:
            enc_val = _escape_path(store.get_value(iter, 0))
            self.configman.remove_option("dirconfig", enc_val)
            self.isConfigChanged()
            store.remove(iter)

    def on_dest_toggled(self, *args):
        if self.widgets["dest2"].get_active():
            self.__enable_target_option("local")

        elif self.widgets["dest3"].get_active():
            self.__enable_target_option("remote")

        # set changed path to destination
        if self.__destination_uri_obj is not None:
            _uri = self.__destination_uri_obj.uri
            self.configman.set("general", "target", _uri)
            self.isConfigChanged()

        else:
            raise ValueError("Unexpected widget was toggled.")

    def __enable_schedule_page(self, enable = True):
        """Enables resp. disables the complete schedule page including the
        tab label.
        
        @param enable: If True the page gets enabled, if False disabled.
        
        """
        self.widgets["label_schedule_page"].set_sensitive(enable)

        self.widgets["rdbtn_no_schedule"].set_sensitive(enable)
        self.widgets["rdbtn_simple_schedule"].set_sensitive(enable)
        self.widgets["rdbtn_custom_schedule"].set_sensitive(enable)

        self.widgets["label_simple_schedule_freq"].set_sensitive(enable)
        self.widgets["cmbbx_simple_schedule_freq"].set_sensitive(enable)

        self.widgets["label_custom_cronline"].set_sensitive(enable)
        self.widgets["txtfld_custom_cronline"].set_sensitive(enable)

        self.widgets["img_schedule_infotext"].set_sensitive(enable)
        self.widgets["label_schedule_infotext"].set_sensitive(enable)

    def __enable_schedule_option(self, option):
        """Enables resp. disables the according widgets for the given schedule
        option. Values are not set.
        """
        def __enable_no_scheduling(enable = True):
            """Enables resp. disables options related to 'No scheduling'.
            Values are not set.
            """
            pass

        def __enable_simple_scheduling(enable = True):
            """Enables resp. disables options related to 'Simple scheduling'.
            Values are not set.
            """
            self.widgets["label_simple_schedule_freq"].set_sensitive(enable)
            self.widgets["cmbbx_simple_schedule_freq"].set_sensitive(enable)

        def __enable_custom_scheduling(enable = True):
            """Enables resp. disables options related to 'Custom scheduling'.
            Values are not set.
            """
            self.widgets["label_custom_cronline"].set_sensitive(enable)
            self.widgets["txtfld_custom_cronline"].set_sensitive(enable)

        if option == "no":
            __enable_no_scheduling(enable = True)
            __enable_simple_scheduling(enable = False)
            __enable_custom_scheduling(enable = False)
        elif option == "simple":
            __enable_no_scheduling(enable = False)
            __enable_simple_scheduling(enable = True)
            __enable_custom_scheduling(enable = False)
        elif option == "custom":
            __enable_no_scheduling(enable = False)
            __enable_simple_scheduling(enable = False)
            __enable_custom_scheduling(enable = True)
        else:
            raise ValueError("Unknown schedule option given.")

    def __set_schedule_option(self, option):
        self.__enable_schedule_option(option)
        if option == "no":
            self.widgets["rdbtn_no_schedule"].set_active(is_active = True)
        elif option == "simple":
            self.widgets["rdbtn_simple_schedule"].set_active(is_active = True)
        elif option == "custom":
            self.widgets["rdbtn_custom_schedule"].set_active(is_active = True)
        else:
            raise ValueError("Unknown schedule option given.")

    def __set_value_txtfld_custom_cronline(self, cronline):
        self.widgets['txtfld_custom_cronline'].set_text(cronline)

    def __set_value_cmbbx_simple_schedule_freq(self, frequency):
        _valid_freqs = ConfigManagerStaticData.get_simple_schedule_frequencies()

        if frequency in _valid_freqs.keys():
            self.widgets['cmbbx_simple_schedule_freq'].set_active(\
                                                        _valid_freqs[frequency])
        else:
            raise ValueError("Unknown anacron setting found!")

    def on_rdbtn_schedule_toggled(self, *args):
        if self.widgets["rdbtn_no_schedule"].get_active():
            self.logger.debug("NO SCHEDULING selected.")
            self.__enable_schedule_option("no")
            _forcechange = self.configman.remove_schedule()
            self.isConfigChanged(_forcechange)

        elif self.widgets["rdbtn_simple_schedule"].get_active():
            self.logger.debug("SIMPLE SCHEDULING selected.")
            self.__enable_schedule_option("simple")
            self.on_cmbbx_simple_schedule_freq_changed()

        elif self.widgets["rdbtn_custom_schedule"].get_active():
            self.logger.debug("CUSTOM SCHEDULING selected.")
            self.__enable_schedule_option("custom")
            self.on_txtfld_custom_cronline_changed()

        else:
            raise ValueError("Unexpected radio button group member was changed.")

    def on_cmbbx_simple_schedule_freq_changed(self, *args):
        """Signal handler which is called whenever the schedule time frequency
        in the 'time_freq' combo box is changed.
        
        """
        _selection = self.widgets["cmbbx_simple_schedule_freq"].get_active()

        _valid_freqs = ConfigManagerStaticData.get_simple_schedule_frequencies()
        if _selection in _valid_freqs.values():
            for _freq_key in _valid_freqs.keys():
                if _selection == _valid_freqs[_freq_key]:
                    self.configman.setSchedule(0, _freq_key)
        self.isConfigChanged()
        self.logger.debug("Scheduling is: %s" % str(self.configman.get_schedule()))

    def on_txtfld_custom_cronline_changed(self, *args):
        _cronline = self.widgets['txtfld_custom_cronline'].get_text()
#        print "WE MUST CHECK THE INPUT!"
#TODO: WE MUST CHECK THE INPUT!
        self.configman.setSchedule(1, _cronline)
        self.isConfigChanged()
        self.logger.debug("Scheduling is: %s" % str(self.configman.get_schedule()))

    def on_time_maxinc_changed(self, *args):
        """Adds a changed value for 'maximum increment days' to the
        configuration.
        """
        self.configman.set("general", "maxincrement",
                            int(self.widgets["time_maxinc"].get_value()))
        self.isConfigChanged()

    def __enable_purge_options(self, enable = True):
        self.widgets['purgeradiobutton'].set_sensitive(enable)
        self.widgets['purgedays'].set_sensitive(enable)
        self.widgets['purgelabel'].set_sensitive(enable)
        self.widgets['logpurgeradiobutton'].set_sensitive(enable)
        self.widgets['purgelabel2'].set_sensitive(enable)

    def on_purgecheckbox_toggled(self, *args):
        """Signal handler that is called whenever the state of the 'Purging'
        checkbox is toggled.
        """
        purge = self.widgets["purgecheckbox"].get_active()
        if purge:
            self.__enable_purge_options(enable = True)
            self.on_purgeradiobutton_toggled()
        else:
            self.__enable_purge_options(enable = False)
            self.configman.remove_option("general", "purge")
            self.isConfigChanged()

    def on_purgeradiobutton_toggled(self, *args):
        if self.widgets["purgeradiobutton"].get_active():
            self.widgets["purgedays"].set_sensitive(True)
            try: i = int(self.widgets["purgedays"].get_text())
            except: i = -1
            if not (i > 0 and i < 10000):    i = 30
            self.widgets["purgedays"].set_text(str(i))
            self.configman.set("general", "purge", str(i))
            self.isConfigChanged()
        elif self.widgets["logpurgeradiobutton"].get_active():
            self.widgets["purgedays"].set_sensitive(False)
            self.configman.set("general", "purge", "log")
            self.isConfigChanged()

    def on_purgedays_changed(self, *args):
        try:
            i = int(self.widgets["purgedays"].get_text())
        except:
            i = 30
        if not (i > 0 and i < 10000):
            i = 30
        self.configman.set("general", "purge", str(i))
        self.isConfigChanged()

    def __enable_ssl_options(self, enable = True):
        self.widgets['label_certificate'].set_sensitive(enable)
        self.widgets['label_key'].set_sensitive(enable)
        self.widgets['keyfilechooser'].set_sensitive(enable)
        self.widgets['crtfilechooser'].set_sensitive(enable)

    def __get_application_widget(self):
        """Returns the top level application widget object.
        """
        app_obj = self.widgets['sbackupConfApp']
        return app_obj

    def on_testMailButton_clicked(self, *args):
        testmail_res = False
        if self.isConfigChanged(force_the_change = False) is True:
            misc.show_infodialog(parent = self.__get_application_widget(),
                    headline_str = _("Configuration has changed"),
                    message_str = _("There are unsaved modifications. Please save the configuration or revert these changes before testing the mail settings."))
        else:
            try :
                testmail_res = self.configman.testMail()
            except SBException, _exc:
                misc.show_warndialog(parent = self.__get_application_widget(),
                    headline_str = _("Test mail settings"),
                    message_str = _("The test failed with following output:"),
                    secmsg_str = "%s" % (_exc))

            if testmail_res is True:
                misc.show_infodialog(parent = self.__get_application_widget(),
                        headline_str = _("Test mail settings"),
                        message_str = _("The test was successful."))

    def on_smtplogincheckbox_toggled(self, *args):
        if self.widgets['smtplogincheckbox'].get_active():
            self.widgets['smtplogininfo'].set_sensitive(True)
            if self.widgets['smtplogin'].get_text():
                self.configman.set("report", "smtpuser",
                                    self.widgets['smtplogin'].get_text())
            if self.widgets['smtppassword'].get_text() :
                self.configman.set("report", "smtpuser",
                                    self.widgets['smtppassword'].get_text())
        else:
            self.widgets['smtplogininfo'].set_sensitive(False)
            if self.configman.has_option("report", "smtpuser") :
                self.configman.remove_option("report", "smtpuser")
            if self.configman.has_option("report", "smtppassword") :
                self.configman.remove_option("report", "smtppassword")
        self.isConfigChanged()

    def __enable_secure_email_options(self, enable = True):
        self.widgets['TLSinfos'].set_sensitive(enable)

    def on_TLScheckbutton_toggled(self, *args):
        """Signal handler that is called when the checkbutton for
        'secure email connection' is checked/unchecked.
        """
        if self.widgets['TLScheckbutton'].get_active():
            # secure connection is enabled
            self.__enable_secure_email_options(enable = True)
            self.configman.set("report", "smtptls", "1")
        else:
            # *NO* secure connection
            self.__enable_secure_email_options(enable = False)
            if self.configman.has_option("report", "smtptls") :
                self.configman.remove_option("report", "smtptls")
            if self.configman.has_option("report", "smtpcert") :
                self.configman.remove_option("report", "smtpcert")
            if self.configman.has_option("report", "smtpkey") :
                self.configman.remove_option("report", "smtpkey")
        self.isConfigChanged()

    def on_TLSradiobutton_toggled(self, *args):
        if self.widgets['TLSradiobutton'].get_active():
            # TLS (i.e. no cert/key required)
            self.__enable_ssl_options(enable = False)
            if self.configman.has_option("report", "smtpcert"):
                self.configman.remove_option("report", "smtpcert")
            if self.configman.has_option("report", "smtpkey"):
                self.configman.remove_option("report", "smtpkey")
        elif self.widgets['SSLradiobutton'].get_active():
            self.__enable_ssl_options(enable = True)
            if self.widgets['crtfilechooser'].get_filename() :
                self.on_crtfilechooser_selection_changed()
            if self.widgets['keyfilechooser'].get_filename() :
                self.on_keyfilechooser_selection_changed()
        else:
            raise ValueError("Unexpected signal received.")
        self.isConfigChanged()

    def on_ex_addftype_clicked(self, *args):
        """Signal handler that is called when the user presses the
        'Add filetype' exclude button.
        
        @note: A dot separating filename and extension is added automatically.
        """
        _known_ftypes_dict = ConfigManagerStaticData.get_known_ftypes_dict()
        dialog = self.widgets["ftypedialog"]
        response = dialog.run()
        dialog.hide()
        if response == gtk.RESPONSE_OK:
            if self.widgets["ftype_st"].get_active():
                ftype = self.widgets["ftype_box"].get_model()\
                                    [self.widgets["ftype_box"].get_active()][0]
            else:
                ftype = self.widgets["ftype_custom_ex"].get_text()

            r = r""
            if self.configman.has_option("exclude", "regex"):
                r = self.configman.get("exclude", "regex")
# Bugfix LP #258542 
            ftype_regex = r"\.%s$" % ftype.strip()
            _sep = ","
            if _sep in ftype_regex:
                _msg = _("The given expression contains unsupported characters ('%s'). Currently it is not possible to use these characters in exclude expressions.") % _sep
                misc.show_warndialog(message_str = _msg,
                                    parent = self.__get_application_widget(),
                                    headline_str = _("Unsupported character"))
            else:
                if not Util.has_conf_entry(r, ftype_regex):
                    r = Util.add_conf_entry(r, ftype_regex)
                    self.configman.set("exclude", "regex", r)
                    if ftype in _known_ftypes_dict:
                        self.ex_ftype.append([_known_ftypes_dict[ftype], ftype])
                    else:
                        self.ex_ftype.append([_("Custom"), ftype])
        else:
            pass
        self.isConfigChanged()

    def on_ex_delftype_clicked(self, *args):
        """
        @todo: Check whether escaping of value (re.escape(value)) before \
                adding it is required?
        """
        (store, iter) = self.ex_ftypetv.get_selection().get_selected()
        if store and iter:
            value = store.get_value(iter, 1)
            r = self.configman.get("exclude", "regex")
# Bugfix LP #258542 
            ftype_regex = r"\.%s$" % value
            r = Util.remove_conf_entry(r, ftype_regex)
            self.configman.set("exclude", "regex", r)
            self.isConfigChanged()
            store.remove(iter)

    def on_ex_addregex_clicked(self, *args):
        """Signal handler which is called when button 'Add regex' for
        exclusion is clicked.
        """
        dialog = self.widgets["regexdialog"]
        response = dialog.run()
        dialog.hide()
        if response == gtk.RESPONSE_OK:
            regex = self.widgets["regex_box"].get_text()
            _sep = ","
            if _sep in regex:
                _msg = _("The given expression contains unsupported characters ('%s'). Currently it is not possible to use these characters in exclude expressions.") % _sep
                misc.show_warndialog(message_str = _msg,
                                    parent = self.__get_application_widget(),
                                    headline_str = _("Unsupported character"))

            elif Util.is_empty_regexp(regex):
                misc.show_errdialog(parent = self.__get_application_widget(),
                            message_str = \
                _("Empty expression. Please enter a valid regular expression."))
            else:
                if Util.is_valid_regexp(regex):
                    r = r""
                    if self.configman.has_option("exclude", "regex"):
                        r = self.configman.get("exclude", "regex")

                    if not Util.has_conf_entry(r, regex):
                        r = Util.add_conf_entry(r, regex)
                        self.configman.set("exclude", "regex", r)
                        self.ex_regex.append([regex])
                else:
                    misc.show_errdialog(parent = self.__get_application_widget(),
                            message_str = \
                                _("Provided regular expression is not valid."))
        else:
            pass
        self.isConfigChanged()

    def on_ex_delregex_clicked(self, *args):
        (store, iter) = self.ex_regextv.get_selection().get_selected()
        if store and iter:
            value = store.get_value(iter, 0)
            r = self.configman.get("exclude", "regex")
            r = Util.remove_conf_entry(r, value)
            self.configman.set("exclude", "regex", r)
            self.isConfigChanged()
            store.remove(iter)

    def on_includetv_key_press_event(self, widget, event, *args): #IGNORE:W0613
        if event.keyval == gtk.keysyms.Delete :
            self.on_inc_del_clicked()

    def on_ex_pathstv_key_press_event(self, widget, event, *args): #IGNORE:W0613
        if event.keyval == gtk.keysyms.Delete :
            self.on_ex_delpath_clicked()

    def on_ex_ftypetv_key_press_event(self, widget, event, *args): #IGNORE:W0613
        if event.keyval == gtk.keysyms.Delete :
            self.on_ex_delftype_clicked()

    def on_ex_regextv_key_press_event(self, widget, event, *args): #IGNORE:W0613
        if event.keyval == gtk.keysyms.Delete :
            self.on_ex_delregex_clicked()

    def on_ex_max_toggled(self, *args): #IGNORE:W0613
        """Signal handler which is called whenever the checkbutton
        'Do not backup files bigger than' is checked resp. unchecked.
        """
        _exclude_max = self.widgets["ex_max"].get_active()

        if _exclude_max:
            self.widgets["ex_maxsize"].set_sensitive(True)
            self.on_ex_maxsize_changed()
        else:
            self.widgets["ex_maxsize"].set_sensitive(False)
            self.configman.remove_option("exclude", "maxsize")
            self.isConfigChanged()

    def on_ex_maxsize_changed(self, *args): #IGNORE:W0613
        """Signal handler which is called when the value for
        maximum file size is changed. The number (from the text field)
        is interpreted as Megabyte (MB).
        """
        msize = int(self.widgets["ex_maxsize"].get_value())
        self.configman.set("exclude", "maxsize", str(msize * 1024 * 1024))
        self.isConfigChanged()

    def on_followlinks_toggled(self, *args): #IGNORE:W0613
        if self.widgets['followlinks'].get_active():
            self.configman.set("general", "followlinks", 1)
        else :
            self.configman.remove_option("general", "followlinks")
        self.isConfigChanged()

    def on_btn_browse_local_clicked(self, *args): #IGNORE:W0613
        _dest = self.__dest_from_config_helper()
        ctarget = _dest.query_dest_display_name()
        
        self.logger.debug("Current destination: %s" % ctarget)
        self.widgets["dialog_browse_localdest"].set_current_folder(ctarget)
        gobject.idle_add(self.__browse_localdest)

    def __browse_localdest(self):
        dialog = self.widgets["dialog_browse_localdest"]
        dialog.set_transient_for(self.__get_application_widget())
        response = dialog.run()

        if response == gtk.RESPONSE_APPLY:
            dialog.hide()
            _target = dialog.get_filename()            
            if _target is None:  # LP #1174124 Catch invalid selections
                _target = ""
            
            if not (os.path.isdir(_target) and \
                    os.access(_target, os.R_OK | os.W_OK | os.X_OK)):
                _sec_msg = _("Please make sure the directory exists "\
                  "and check file permissions. Then try again.")
                _message_str = _("Selected backup destination folder"\
                  "`%s` does not exist or is not accessable.") % _target
                _headline_str = \
                _("Unable to access selected folder")
                gobject.idle_add(misc.show_errdialog,
                                  _message_str,
                                  self.__get_application_widget(),
                                  _headline_str, _sec_msg)
            else:
                self.widgets['dest_local'].set_text(_target)
                self.configman.set("general", "target", _target)
                _dest = self.__dest_from_config_helper()
                self.isConfigChanged()

        elif response in (gtk.RESPONSE_NONE, gtk.RESPONSE_CANCEL,
                          gtk.RESPONSE_DELETE_EVENT):
            dialog.hide()

        else:
            self.logger.error(_("Unexpected dialog response: %s") % response)
            gobject.idle_add(self.__browse_localdest)

    def on_checkbtn_show_password_toggled(self, *args): #IGNORE:W0613
        self.__set_entry_remote_pass_visibiliy()

    def __set_entry_remote_pass_visibiliy(self):
        _pass_e = self.widgets['entry_set_remote_pass']
        _pass_e.set_visibility(visible = self.widgets['checkbtn_show_password'].get_active())

    def on_btn_set_remote_clicked(self, *args): #IGNORE:W0613
        dest_obj = pathparse.UriParser()
        dest_obj.set_and_parse_uri(uri = self.configman.get_destination_path())
        if dest_obj is not None:
            self.__destination_uri_obj = dest_obj
        gobject.idle_add(self.__show_connect_remote_dialog)

    def __show_connect_remote_dialog(self):
        dialog = self.widgets["dialog_connect_remote"]
        btn_connect = self.widgets['btn_connect_remote']

        _server_e = self.widgets['entry_set_remote_server']
        _port_e = self.widgets['entry_set_remote_port']
        _dir_e = self.widgets['entry_set_remote_dir']
        _user_e = self.widgets['entry_set_remote_user']
        _pass_e = self.widgets['entry_set_remote_pass']
        _service_b = self.widgets['cmb_set_remote_service']

        dialog.set_transient_for(self.__get_application_widget())
        self.__set_entry_remote_pass_visibiliy()
        btn_connect.grab_focus()

        # default values
        _rservice = fam.get_default_remote_service()
        _server = ""
        _port = ""
        _dir = ""
        _user = ""
        _pass = ""

        # fill entries
#TODO: wrap use of UriParser into Fam facade        
        if self.__destination_uri_obj is not None:
            if not self.__destination_uri_obj.is_local():

                _rservice = fam.get_service_from_scheme(self.__destination_uri_obj.uri_scheme)

                if _rservice is None:
                    raise SBException("Unable to query remote service")

                _server = self.__destination_uri_obj.hostname
                _port = self.__destination_uri_obj.port
                _dir = self.__destination_uri_obj.path
                _user = self.__destination_uri_obj.username
                _pass = self.__destination_uri_obj.password


        _servmodel = _service_b.get_model()
        for _idx in range(len(_servmodel)):
            if _servmodel[_idx][misc.MODEL_COLUMN_INDEX_KEY] == _rservice:
                _sidx = _idx
                break

        _service_b.set_active(_sidx)
        _server_e.set_text(_server)
        _port_e.set_text(_port)
        _dir_e.set_text(_dir)
        _user_e.set_text(_user)
        _pass_e.set_text(_pass)

        dialog.set_sensitive(True)
        response = dialog.run()

        if response == gtk.RESPONSE_APPLY:
            self.logger.info(_("Connect to remote destination"))

            _sidx = _service_b.get_active()
            _rservice = _servmodel[_sidx][misc.MODEL_COLUMN_INDEX_KEY]

            _server = _server_e.get_text()
            _port = _port_e.get_text()
            _dir = _dir_e.get_text()
            _user = _user_e.get_text()
            _pass = _pass_e.get_text()

            self.logger.debug("User input in dialog 'connect to host'")
            self.logger.debug("  Service: %s" % _rservice)
            self.logger.debug("  Server: %s" % _server)
            self.logger.debug("  Port: %s" % _port)
            self.logger.debug("  Path: %s" % _dir)
            self.logger.debug("  User: %s" % _user)
            self.logger.debug("  Pass: %s\n" % ("*" * len(_pass)))

            dialog.set_sensitive(False)
            misc.set_watch_cursor(dialog)

            _scheme = fam.get_scheme_from_service(_rservice)
            _uri = pathparse.construct_remote_uri_from_tupel(_scheme, _server, _port, _dir, _user, _pass)
            self.__destination_uri_obj.set_and_parse_uri(_uri)

            self.__destination_hdl = fam.get_fam_target_handler_facade_instance()
            self.__destination_hdl.set_initialize_callback(self._mount_done_cb)
            self.__destination_hdl.set_destination(self.__destination_uri_obj.uri)
            self.__destination_failure = False

            gobject.idle_add(self.__destination_hdl.initialize)

        elif response == gtk.RESPONSE_CANCEL or \
             response == gtk.RESPONSE_DELETE_EVENT:
            dialog.hide()
            _icon = self.widgets["dest_remote_light"]
            _icon.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_MENU)

        else:
            self.logger.error(_("Unexpected dialog response: %s") % response)
            gobject.idle_add(self.__show_connect_remote_dialog)

    def _mount_done_cb(self, error):
        self.logger.debug("GUI._mount_done_cb: error: %s" % str(error))
        dialog = self.widgets["dialog_connect_remote"]

        if error is None:
            self.logger.info(_("Mount was sucessful (no errors)"))
            gobject.idle_add(self._do_remote_tests)
        else:
            misc.unset_cursor(dialog)

            _icon = self.widgets["dest_remote_light"]
            _icon.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_MENU)

            misc.show_warndialog(
                parent = self.__get_application_widget(),
                message_str = str(error),
                headline_str = _("Unable to mount host"))
            self.__destination_failure = True
            gobject.idle_add(self.__show_connect_remote_dialog)

    def _umount_done_cb(self, error):
        self.logger.debug("GUI._umount_done_cb: error: %s" % str(error))
        dialog = self.widgets["dialog_connect_remote"]

        if error is None:
            self.logger.info(_("Umount was sucessful (no errors)"))
        else:
            misc.unset_cursor(dialog)

            _icon = self.widgets["dest_remote_light"]
            _icon.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_MENU)

            misc.show_warndialog(
                parent = self.__get_application_widget(),
                message_str = str(error),
                headline_str = _("Unable to unmount host"))
            self.__destination_failure = True
        gobject.idle_add(self._umount_done)

    def _umount_done(self):
        self.logger.debug("umount done - failures: %s" % self.__destination_failure)
        dialog = self.widgets["dialog_connect_remote"]
        misc.unset_cursor(dialog)
        dialog.set_sensitive(True)
        if self.__destination_failure == False:
            self.__destination_hdl = None
            dialog.hide()

            _uri = self.__destination_uri_obj.uri
            _displname = self.__destination_uri_obj.query_display_name()
            self.configman.set("general", "target", _uri)
            self.widgets['dest_remote'].set_text(_displname)

            _icon = self.widgets["dest_remote_light"]
            _icon.set_from_stock(gtk.STOCK_OK, gtk.ICON_SIZE_MENU)

            self.isConfigChanged()
        else:
            _icon = self.widgets["dest_remote_light"]
            _icon.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_MENU)

            gobject.idle_add(self.__show_connect_remote_dialog)

    def _do_remote_tests(self):
        error = None
        self.logger.info(_("Perfom tests on remote host"))
        dialog = self.widgets["dialog_connect_remote"]
        try:
            self.__destination_hdl.test_destination()
        except exceptions.RemoteMountTestFailedError, error:
            misc.unset_cursor(dialog)

            _icon = self.widgets["dest_remote_light"]
            _icon.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_MENU)

            misc.show_warndialog(
                parent = self.__get_application_widget(),
                message_str = str(error),
                headline_str = _("Unable to access remote destination"))
            misc.set_watch_cursor(dialog)
            self.__destination_failure = True
        else:
            self.logger.info(_("All tests passed"))

        self.__destination_hdl.set_terminate_callback(self._umount_done_cb)
        gobject.idle_add(self.__destination_hdl.terminate)


    def on_logfilechooser_selection_changed(self, *args): #IGNORE:W0613
        self.configman.set_logdir(self.widgets['logfilechooser'].get_filename())
        self.configman.set_logfile_templ_to_config()
        self.isConfigChanged()
        self.logger.debug("Log file set: " + self.configman.get("log", "file"))

    def on_loglevelcombobox_changed(self, *args): #IGNORE:W0613
        if self.widgets['loglevelcombobox'].get_active_text() == "Info" :
            self.configman.set("log", "level", "20")
            self.isConfigChanged()
            self.logger.debug("Log level : " + self.configman.get("log", "level"))
        elif self.widgets['loglevelcombobox'].get_active_text() == "Debug" :
            self.configman.set("log", "level", "10")
            self.isConfigChanged()
            self.logger.debug("Log level : " + self.configman.get("log", "level"))
        elif self.widgets['loglevelcombobox'].get_active_text() == "Error" :
            self.configman.set("log", "level", "40")
            self.isConfigChanged()
            self.logger.debug("Log level : " + self.configman.get("log", "level"))
        elif self.widgets['loglevelcombobox'].get_active_text() == "Warning" :
            self.configman.set("log", "level", "30")
            self.isConfigChanged()
            self.logger.debug("Log level : " + self.configman.get("log", "level"))

    def on_smtpfrom_changed(self, *args): #IGNORE:W0613
        self.__check_for_section("report")
        if self.widgets['smtpfrom'].get_text() != "":
            self.configman.set("report", "from", self.widgets['smtpfrom'].get_text())
            self.isConfigChanged()
        else :
            self.configman.remove_option("report", "from")
            self.isConfigChanged()

    def on_smtpto_changed(self, *args): #IGNORE:W0613
        self.__check_for_section("report")
        if self.widgets['smtpto'].get_text() != "":
            self.configman.set("report", "to", self.widgets['smtpto'].get_text())
            self.isConfigChanged()
        else :
            self.configman.remove_option("report", "to")
            self.isConfigChanged()

    def on_smtpserver_changed(self, *args): #IGNORE:W0613
        self.__check_for_section("report")
        if self.widgets['smtpserver'].get_text() != "":
            self.configman.set("report", "smtpserver", self.widgets['smtpserver'].get_text())
            self.isConfigChanged()
        else :
            self.configman.remove_option("report", "smtpserver")
            self.isConfigChanged()

    def on_smtpport_changed(self, *args):
        self.__check_for_section("report")
        if self.widgets['smtpport'].get_text() != "":
            self.configman.set("report", "smtpport", self.widgets['smtpport'].get_text())
            self.isConfigChanged()
        else :
            self.configman.remove_option("report", "smtpport")
            self.isConfigChanged()

    def on_smtplogin_changed(self, *args):
        self.__check_for_section("report")
        if self.widgets['smtplogin'].get_text() != "":
            self.configman.set("report", "smtpuser", self.widgets['smtplogin'].get_text())
            self.isConfigChanged()
        else :
            self.configman.remove_option("report", "smtpuser")
            self.isConfigChanged()

    def on_smtppassword_changed(self, *args):
        self.__check_for_section("report")
        if self.widgets['smtppassword'].get_text() != "":
            self.configman.set("report", "smtppassword", self.widgets['smtppassword'].get_text())
            self.isConfigChanged()
        else :
            self.configman.remove_option("report", "smtppassword")
            self.isConfigChanged()

    def on_crtfilechooser_selection_changed(self, *args):
        self.__check_for_section("report")
        smtpcert = self.widgets['crtfilechooser'].get_filename()
        if smtpcert != None and os.path.isfile(smtpcert):
            self.configman.set("report", "smtpcert", self.widgets['crtfilechooser'].get_filename())
            self.isConfigChanged()
            self.logger.debug("Certificate : " + str(self.configman.get("report", "smtpcert")))

    def on_keyfilechooser_selection_changed(self, *args):
        self.__check_for_section("report")
        smtpkey = self.widgets['keyfilechooser'].get_filename()
        if smtpkey != None and os.path.isfile(smtpkey):
            self.configman.set("report", "smtpkey", smtpkey)
            self.isConfigChanged()
            self.logger.debug("Key : " + str(self.configman.get("report", "smtpkey")))

    def on_sbackupConfApp_delete_event(self, *args):
        """Signal handler that is called when the window decorator close
        button is clicked.
        """
        cancelled = self.ask_save_config()
        return cancelled

    def __terminate_app(self):
        self.configman = None
        self.orig_configman = None
        gtk.main_quit()

    def on_sbackupConfApp_destroy(self, *args):
        """Signal handler that is called when the window was destroyed.
        """
        self.__terminate_app()

    def on_exit_activate(self, *args):
        """Signal handler that is called when the 'Quit' menu item
        is selected.
        """
        cancelled = self.ask_save_config()
        if not cancelled:
            self.__terminate_app()

    def on_ftype_custom_ex_changed(self, *args):
#TODO: on_ftype_custom_ex_changed
#        print("TODO: on_ftype_custom_ex_changed")
        pass

    def on_prfManager_activate(self, *args):
        """Launch Profile manager dialog
        """
        cancelled = self.ask_save_config()
        if not cancelled:
            dialog = self.widgets["ProfileManagerDialog"]
            dialog.run()
            dialog.hide()

    def on_addProfileButton_clicked(self, *args):
        valid_input = False
        prf_set = False

        prfDir = local_file_utils.normpath(self.__configFileHandler.get_user_confdir(), "sbackup.d")
        if not os.path.exists(prfDir):
            os.makedirs(prfDir)

        dialog = self.widgets['askNewPrfNameDialog']
        dialog.set_title("")

        while not valid_input:
            response = dialog.run()
            dialog.hide()

            if response == gtk.RESPONSE_OK :
                enable = self.widgets['enableNewPrfCB'].get_active()
                prfName = self.widgets['newPrfNameEntry'].get_text()
                prfName = prfName.strip()
                prfConf = local_file_utils.normpath(prfDir, "sbackup-%s.conf" % prfName)
                prfConfDisabled = "%s-disable" % prfConf

                if not prfName or prfName is '':
                    misc.show_warndialog(
                        parent = self.widgets["ProfileManagerDialog"],
                        message_str = _("The given name of the new profile is empty. Please enter a valid profile name."),
                        headline_str = _("Profile name not valid"))
                    continue

                if os.path.exists(prfConf) or os.path.exists(prfConfDisabled):
                    misc.show_warndialog(
                        parent = self.widgets["ProfileManagerDialog"],
                        message_str = _("The given name of the new profile already exists. Please enter another name."),
                        headline_str = _("Profile name not valid"),
                        secmsg_str = _("Renaming of profiles is not supported."))
                    continue
                # if we reach this branch a valid profile name was choosen
                prf_set = True
            # if this branch is reached the input (OK, Cancel, Destroy) was
            # valid
            valid_input = True
        # end of while loop

        if prf_set:
            self.logger.debug("Got new profile name '%s : enable=%r' " % (prfName, enable))
            if not enable:
                prfConf = prfConfDisabled
            confman = ConfigManager.ConfigManager()
            confman.saveConf(prfConf)
            self.profiles.append([enable, prfName, prfConf])
#        else:
#            print "Adding of profile canceled."

    def on_removeProfileButton_clicked(self, *args):
        tm, iter = self.profilestv.get_selection().get_selected()

        if iter is None:
            _show_errmsg_no_profile_selected()
            return

        prfName, prfConf = tm.get_value(iter, 1), tm.get_value(iter, 2)
        if prfName == ConfigManagerStaticData.get_default_profilename():
            _forbid_default_profile_removal()
        else :
            warning = _("<b>Delete configuration profile?</b>\n\nYou are trying to remove a configuration profile. You will not be able to restore it. If you are not sure, use the 'enable|disable' functionality instead.\n\nDo you really want to delete the profile '%(name)s'?") % {'name': glib.markup_escape_text(prfName)}
            dialog = gtk.MessageDialog(type = gtk.MESSAGE_WARNING, flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, buttons = gtk.BUTTONS_YES_NO)
            dialog.set_markup(warning)
            response = dialog.run()
            dialog.destroy()

            if response == gtk.RESPONSE_YES :
                self.logger.debug("Remove Profile '%s' configuration" % prfName)
                if os.path.exists(prfConf) :
                    os.remove(prfConf)
                self.profiles.remove(iter)

            elif response == gtk.RESPONSE_NO :
                pass

    def on_editProfileButton_clicked(self, *args):
        tm, iter = self.profilestv.get_selection().get_selected()
        if iter is None:
            _show_errmsg_no_profile_selected()
            return
        prfName, prfConf = tm.get_value(iter, 1), tm.get_value(iter, 2)
        self.logger.debug("Load Profile '%s' configuration" % prfName)

        self.conffile = prfConf
        self.on_reload_clicked()
        self.widgets["ProfileManagerDialog"].hide()

    def on_closeProfileManagerButton_clicked(self, *args):
        """
        Load the default configuration file
        """
        self.logger.debug("Load the default configuration file '%s'" % self.default_conffile)
        self.conffile = self.default_conffile
        self.on_reload_clicked()

    def on_prfEnableCB_toggled(self, *args):

        tm, iter = self.profilestv.get_selection().get_selected()
        if iter is None:
            _show_errmsg_no_profile_selected()
            return
        enable, prfName, prfConf = tm.get_value(iter, 0), tm.get_value(iter, 1), tm.get_value(iter, 2)

        if prfName == ConfigManagerStaticData.get_default_profilename():
            _forbid_default_profile_disable()
        else :
            dir, file = prfConf.rsplit(os.sep, 1)

            # rename the file 
            if enable :
                # then disable
                self.logger.debug("Disabling %s " % prfName)
                os.rename(prfConf, prfConf + "-disable")
                self.profiles.set_value(iter, 0, False)
                self.profiles.set_value(iter, 2, prfConf + "-disable")

            else :
                # enable it
                self.logger.debug("Enabling %s " % prfName)
                os.rename(prfConf, prfConf.rstrip("-disable"))
                self.profiles.set_value(iter, 0, True)
                self.profiles.set_value(iter, 2, prfConf.rstrip("-disable"))

    def on_menu_set_default_settings_activate(self, *args):
        """Signal handler which is activated when the user either selects
        the menu item 'Set default settings...' or clicks the according
        toolbar button. The method presents a message box where
        confirmation for the changes is required.
        """
        dialog = self.widgets["dialog_default_settings"]
        label = self.widgets["label_dialog_default_settings_content"]
        btn_cancel = self.widgets['btn_cancel_default_settings']

        txt = _("<big><b>Set default values for current profile?</b></big>\nThis will restore the default values for the profile currently edited: '%s'.\n\nThese predefined settings are recommended for most users. Check whether they are appropriate for your use before saving the changed configuration.") % glib.markup_escape_text(self.configman.getProfileName())

        label.set_line_wrap(True)
        label.set_markup(txt)
        misc.label_set_autowrap(label)
        btn_cancel.grab_focus()

        response = dialog.run()
        dialog.hide()
        if response == gtk.RESPONSE_APPLY:
            self.logger.info(_("Default settings are being applied."))
            self._set_default_settings()
        elif response == gtk.RESPONSE_CANCEL or \
             response == gtk.RESPONSE_DELETE_EVENT:
            pass
        else:
            self.logger.error(_("Unexpected dialog response: %s") % response)
            raise ValueError("Unexpected dialog response: %s" % response)

    def _set_default_settings(self):
        """Sets default values (which might be considered as recommended
        values for some usecase) for the current profile.
        """
        # implementation note: the values are set in the configuration
        # manager and afterwards the according UI widgets are updated
        # with these new values.
        self.configman.set_values_to_default()
        # filesystem is not probed since we want to apply *new* values
        self._fill_widgets_from_config(probe_fs = False)


def _forbid_default_profile_removal():
    """Helper function that shows an info box which states that we are
    not able to do the given action on the default profile.    
    """
    info = _("<b>Unable to remove default profile</b>\n\nThe default profile cannot be removed. In the case you want to use just a single profile, please set up the default profile accordingly.")

    dialog = gtk.MessageDialog(type = gtk.MESSAGE_INFO,
                    flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                    buttons = gtk.BUTTONS_CLOSE)
    dialog.set_markup(info)
    dialog.run()
    dialog.destroy()


def _forbid_default_profile_disable():
    """Helper function that shows an info box which states that we are
    not able to do the given action on the default profile.    
    """
    info = _("<b>Unable to remove default profile</b>\n\nThe default profile cannot be disabled. In the case you want to use just a single profile, please set up the default profile accordingly.")

    dialog = gtk.MessageDialog(type = gtk.MESSAGE_INFO,
                    flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                    buttons = gtk.BUTTONS_CLOSE)
    dialog.set_markup(info)
    dialog.run()
    dialog.destroy()


def _notify_new_default_profile_created():
    """Helper function that shows an info box which states that
    a new default profile was created.    
    """
    info = _("<b>No backup profile found.</b>\n\nNo default profile was found. You are probably running Simple Backup for the first time. A backup profile using default values was created.\n\nPlease modify the settings according to your needs and save the configuration in order to use it.")

    dialog = gtk.MessageDialog(type = gtk.MESSAGE_INFO,
                    flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                    buttons = gtk.BUTTONS_CLOSE)
    dialog.set_markup(info)
    dialog.run()
    dialog.destroy()


def _show_errmsg_no_profile_selected():
    dialog = gtk.MessageDialog(type = gtk.MESSAGE_ERROR,
                               flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                               buttons = gtk.BUTTONS_CLOSE, message_format = _("Please select a profile."))
    dialog.run()
    dialog.destroy()


def _prepare_dirname(path):
    _res = "%s%s" % (path.rstrip(os.sep), os.sep)
    return _res


def _prepare_filename(path):
    _res = "%s" % (path.rstrip(os.sep))
    return _res


def _escape_path(path):
    _enc_res = path.replace("=", "\\x3d")
    return _enc_res


def main(argv):
    window = SBconfigGTK()
    window.show()
    gtk.main()
    log.shutdown_logging()



#   Simple Backup - pathname manipulation on string basis
#
#   Copyright (c)2010: Jean-Peer Lorenz <peer.loz@gmx.net>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
"""
:mod:`sbackup.util.pathparse` -- pathname manipulation on string basis
========================================================================

.. module:: pathparse
   :synopsis: pathname manipulation on string basis without file access
.. moduleauthor:: Jean-Peer Lorenz <peer.loz@gmx.net>

"""


import urlparse
import urllib
import datetime
import types


from sbackup.util import log
from sbackup.util import system
from sbackup.util import exceptions


#TODO: Do we need to implement this module per backend?


# uri schemes are used in legacy configuration files
# we distinct between those two because of ssh/sftp
# we use ssh and sftp separate: leave the decision to the users
URI_SCHEME_LOCAL_FILE = 'file'
URI_SCHEME_FTP = 'ftp'
URI_SCHEME_SFTP = 'sftp'
URI_SCHEME_SSH = 'ssh'
# davs support currently disabled due to umount issues
#URI_SCHEME_DAVS = 'davs'
URI_SCHEME_NFS = 'nfs'
URI_SCHEME_SMB = 'smb'


VALID_URI_SCHEMES = [ URI_SCHEME_LOCAL_FILE, URI_SCHEME_FTP, URI_SCHEME_SFTP,
                      URI_SCHEME_SSH, URI_SCHEME_NFS,
                      URI_SCHEME_SMB
#                      URI_SCHEME_DAVS
                    ]

# effective uri schemes are used for actual mount process of remote hosts
URI_SCHEME_EFF_LOCAL_FILE = URI_SCHEME_LOCAL_FILE
URI_SCHEME_EFF_FTP = URI_SCHEME_FTP
#URI_SCHEME_EFF_DAVS = URI_SCHEME_DAVS

URI_SCHEME_EFF_SFTP = URI_SCHEME_SFTP
URI_SCHEME_EFF_SSH = URI_SCHEME_SSH
URI_SCHEME_EFF_NFS = URI_SCHEME_NFS
URI_SCHEME_EFF_SMB = URI_SCHEME_SMB


VALID_EFF_URI_SCHEMES = [ URI_SCHEME_EFF_LOCAL_FILE, URI_SCHEME_EFF_FTP, URI_SCHEME_EFF_SFTP,
                          URI_SCHEME_EFF_SSH, URI_SCHEME_EFF_NFS,
                          URI_SCHEME_EFF_SMB
#                          URI_SCHEME_EFF_DAVS
                        ]

URI_SCHEME_TO_EFF_URI_SCHEME = { URI_SCHEME_LOCAL_FILE : URI_SCHEME_EFF_LOCAL_FILE,
                                 URI_SCHEME_FTP : URI_SCHEME_EFF_FTP,
                                 URI_SCHEME_SFTP : URI_SCHEME_EFF_SFTP,
                                 URI_SCHEME_SSH : URI_SCHEME_EFF_SSH,
                                 URI_SCHEME_NFS : URI_SCHEME_EFF_NFS,
                                 URI_SCHEME_SMB : URI_SCHEME_EFF_SMB
#                                 URI_SCHEME_DAVS : URI_SCHEME_EFF_DAVS
                               }


class UriParser(object):
    def __init__(self):
        # the full destination uri i.e. uri scheme + server + path/directory as originally given
        self.__uri = ""
        self.__uri_scheme = "file"
        self.__eff_scheme = ""
        self.__hostname = ""
        self.__path = ""
        self.__port = None
        self.__username = None
        self.__password = None
        self.__clear()

    @property
    def uri(self):
        return self.__uri

    @property
    def uri_scheme(self):
        """Get current URI scheme retrieved from given URI. This is not the effective scheme."""
        return self.__uri_scheme

    @property
    def hostname(self):
        return self.__hostname

    @property
    def path(self):
        return self.__path

    @property
    def port(self):
        _port = self.__port
        if _port is None:
            _port = ""
        return str(_port)

    @property
    def username(self):
        _user = self.__username
        if _user is None:
            _user = ""
        return _user

    @property
    def password(self):
        _pass = self.__password
        if _pass is None:
            _pass = ""
        return _pass

    def __clear(self):
        self.__uri = ""
        self.__uri_scheme = "file"
        self.__eff_scheme = ""
        self.__hostname = ""
        self.__path = ""
        self.__port = None
        self.__username = None
        self.__password = None

    def __str__(self):
        if self.__password is None:
            _pass = "None"
        else:
            _pass = "*" * len(self.__password)
        _res = [ #"URI: `%s`" % self.__uri,
                 "Display name: %s" % self.query_display_name(),
                 "URI scheme: %s" % self.__uri_scheme,
                 "eff. scheme: %s" % self.__eff_scheme,
                 "Hostname: `%s`" % self.__hostname,
                 "Path: `%s`" % self.__path,
                 "Port: %s" % self.__port,
                 "Username: `%s`" % self.__username,
                 "Password: `%s`" % _pass
               ]
        _res_str = "\n".join(_res)
        return _res_str

    def set_and_parse_uri(self, uri):
        _logger = log.LogFactory.getLogger()
#        contains password in plain text
#        _logger.debug("set and parse uri: %s" % uri)
        self.__clear()
        splituri = urlparse.urlsplit(uri, scheme = "", allow_fragments = False)
        self.__uri = splituri.geturl()  # gets normalized URI
        splituri = urlparse.urlsplit(self.__uri, scheme = "", allow_fragments = False)

#        print "\nSplit uri:\n"
#        print "url: %s" % splituri.geturl()
#        print "scheme: %s" % splituri.scheme        #IGNORE:E1103
#        print "netloc: %s" % splituri.netloc        #IGNORE:E1103
#        print "path: %s" % splituri.path            #IGNORE:E1103
#        print "query: %s" % splituri.query          #IGNORE:E1103
#        print "fragment: %s" % splituri.fragment    #IGNORE:E1103
#        print "username: %s" % splituri.username
#        print "password: ????"
#        print "hostname: %s" % splituri.hostname
#        print "port: %s" % splituri.port
#        print "-------------------------"

        assert splituri.fragment == "" #IGNORE:E1103
        assert splituri.query == "" #IGNORE:E1103

        #LP #670646 ignore errors parsing port
        try:
            if splituri.port is not None:
                self.__port = splituri.port
        except ValueError:  
            pass
            
        if splituri.username is not None:
            self.__username = urllib.unquote(splituri.username)

        if splituri.password is not None:
            self.__password = urllib.unquote(splituri.password)

        if splituri.hostname is None:
            self.__hostname = ""
        else:
            self.__hostname = splituri.hostname

        self.__path = urllib.quote(splituri.path) #IGNORE:E1103
#        self.__uri_scheme = splituri.scheme #IGNORE:E1103
        if splituri.scheme == "" and self.__uri.startswith("/"):    #IGNORE:E1103
            self.__uri_scheme = "file"
        else:
            self.__uri_scheme = splituri.scheme                     #IGNORE:E1103

        self.__eff_scheme = _get_eff_uri_scheme(self.__uri_scheme)
        _logger.debug("UriParser:\n%s" % self)

    def __construct_display_name(self):
        _path = urllib.unquote(self.__path)
        if self.is_local():
            _res = _path
        else:
            _user = ""
            _host = ""
            if self.__username is not None:
                _user = "%s@" % self.__username
            if self.__hostname is not None:
                _host = "%s" % self.__hostname
            _res = "%s://%s%s%s" % (self.__uri_scheme, _user, _host, _path)
        return _res

    def query_mount_uri(self):
        """
        Include user in uri being mounted to maintain compliance with Nautilus mounts.
        """
        _logger = log.LogFactory.getLogger()
        _host = ""
        if self.__hostname is not None:
            _host = "%s" % self.__hostname.rstrip("/")

        _user = ""
        if self.__username is not None:
            _user = "%s@" % urllib.quote(self.__username, "")

        _path = self.__path.strip("/")
        if _path != "":
            _path = "/%s" % _path

        _port = ""
        if self.__port is not None:
            _port = ":%s" % self.__port

        _res = "%s://%s%s%s%s" % (self.__eff_scheme, _user, _host, _port, _path)
        _logger.debug("get_mount_uri: %s" % _res)
        return _res

    def is_local(self):
        _local = False
        if self.__eff_scheme == "":
            raise ValueError("Effective scheme not set")
        if self.__eff_scheme == URI_SCHEME_EFF_LOCAL_FILE:
            _local = True
        return _local

    def query_display_name(self):
        return self.__construct_display_name()


def construct_remote_uri_from_tupel(scheme, hostname, port, path, username, password):
    _logger = log.LogFactory.getLogger()
    if not isinstance(scheme, types.StringTypes):
        raise TypeError
    if not isinstance(hostname, types.StringTypes):
        raise TypeError
    if not isinstance(port, types.StringTypes):
        raise TypeError
    if not isinstance(path, types.StringTypes):
        raise TypeError
    if not isinstance(username, types.StringTypes):
        raise TypeError
    if not isinstance(password, types.StringTypes):
        raise TypeError

    if scheme not in VALID_URI_SCHEMES:
        raise ValueError

    # Within the user and password field, any ":", "@", or "/" must be encoded.
    _host = hostname.rstrip("/")
    _user = urllib.quote(username, "")
    _pass = ""
    _port = ""
    _path = path.strip("/")

    if _path != "":
        _path = "/%s" % _path

    if password != "":
        _pass = urllib.quote(password, "")
        _pass = ":%s" % _pass

    if _user != "" or _pass != "":
        _host = "@%s" % hostname

    if port != "":
        _port = ":%s" % port

    _res = "%s://%s%s%s%s%s" % (scheme, _user, _pass, _host, _port, _path)
    _logger.debug("construct_from_remote_tupel: %s" % _res)
    return _res


#def _get_eff_uri_scheme(uri, uri_scheme):
def _get_eff_uri_scheme(uri_scheme):
    """
    :todo: can we construct the eff. scheme manually? aparently ssh is just hardlinked to sftp. 
    """
# following does not work as expected!
#    _gfileobj = _GIOFILE(uri)
#    _uri_scheme = _gfileobj.get_uri_scheme()
#    _has = _gfileobj.has_uri_scheme(_uri_scheme)
#    if _has != True:
#        raise ValueError("URI scheme `%s` not supported by backend" % _uri_scheme)

    if uri_scheme not in VALID_URI_SCHEMES:
        raise exceptions.FileAccessException("URI scheme `%s` not supported by Simple Backup" % uri_scheme)

    _eff_scheme = URI_SCHEME_TO_EFF_URI_SCHEME[uri_scheme]

    if _eff_scheme not in VALID_EFF_URI_SCHEMES:
        raise exceptions.FileAccessException("Eff. URI scheme `%s` not supported by Simple Backup"\
                                             % _eff_scheme)

    return _eff_scheme


def append_time_to_filename(filename, filetype = ""):
    if not isinstance(filename, types.StringTypes):
        raise TypeError("Expected string. Got %s instead." % type(filename))
    if not isinstance(filetype, types.StringTypes):
        raise TypeError("Expected string as file type. Got %s instead." % type(filetype))
    if filetype != "" and not filetype.startswith("."):
        raise ValueError("Given file type must start with dot (.xyz).")

    _time = datetime.datetime.now().isoformat("_").replace(":", ".")
    _res = append_str_to_filename(filename, _time, filetype)
    return _res


def append_str_to_filename(filename, str_to_append, filetype = ""):
    """If a file type (i.e. file extension) is specified, the string
    to append is put in front of the file type extension.
    
    Example: string to append = 123
             filename = basename.log
             result without specifying a filetype = basename.log.123 
             result with specifying filetype '.log' = basename.123.log 
             
    """
    if not isinstance(filename, types.StringTypes):
        raise TypeError("Expected string as filename. Got %s instead." % type(filename))
    if not isinstance(str_to_append, types.StringTypes):
        raise TypeError("Expected string to append. Got %s instead." % type(str_to_append))
    if not isinstance(filetype, types.StringTypes):
        raise TypeError("Expected string as file type. Got %s instead." % type(filetype))
    if filetype != "" and not filetype.startswith("."):
        raise ValueError("Given file type must start with dot (.xyz).")

    _filen = filename
    _ext = ""
    if filetype != "":
        if filename.endswith(filetype):
            _filen = filename.rstrip(filetype)
    _res = "%s.%s%s" % (_filen, str_to_append, filetype)
    return _res


def remove_trailing_sep(path):
    spath = path.rstrip(system.PATHSEP)
    return spath


def remove_leading_sep(path):
    spath = path.lstrip(system.PATHSEP)
    return spath


def ensure_leading_sep(path):
    spath = "%s%s" % (system.PATHSEP, remove_leading_sep(path))
    return spath


def joinpath(*args):
    """Concatenates given paths (i.e. concatenates them and removes trailing
    separators).
    """
    if len(args) == 0:
        _path = system.PATHSEP
    else:
        _path = ""
        for _arg in args:
            _path = "%s%s%s" % (_path, system.PATHSEP, remove_leading_sep(_arg))

        if not args[0].startswith(system.PATHSEP):
            _path = remove_leading_sep(_path)

        _path = remove_trailing_sep(_path)
    return _path



#   Simple Backup - Target file access management using Fuse plugins
#
#   Copyright (c)2009-2010: Jean-Peer Lorenz <peer.loz@gmx.net>
#   Copyright (c)2007-2009: Ouattara Oumar Aziz <wattazoum@gmail.com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#


from gettext import gettext as _
import os
import time
import uuid


from sbackup.core import ConfigManager
from sbackup.fuse_plugins import PluginManager
from sbackup.util.exceptions import SBException
from sbackup.util import exceptions
from sbackup.util import interfaces
from sbackup.util import local_file_utils
from sbackup.util import log
from sbackup.util import pathparse


class  FuseTargetHandler(interfaces.ITargetHandler):
    """
    The Fuse File access Manager
    """

    def __init__(self):
        """
        Constructor
        @param configManager: 
        """
        interfaces.ITargetHandler.__init__(self)

        self._logger = log.LogFactory.getLogger()

        # destination/target specific instances
        self._dest = None
        self._eff_path = None
        self._configuration = None
        self._initialize_callback = None
        self._terminate_callback = None
        self._is_initialized = False

        ## This is the base directory where all mountpoints of remote sites will be located
        _defaults = ConfigManager.get_default_config_obj()
        self.__mountdir = _defaults.get_mountdir()

        ## the list of all mounted dirs , should be filled by initialize.
        # It's a dict with key = remoteSource and value = mountpoint
        self.__mountedDirs = {}

    def set_use_mainloop(self, use):
        pass

    def set_destination(self, path):
        _dest_obj = pathparse.UriParser()
        _dest_obj.set_and_parse_uri(uri = path)
        self._dest = _dest_obj
        if self._dest.is_local():
            self._eff_path = self._dest.uri

    def get_destination(self):
        _path = self._dest.uri
        return _path

    def set_initialize_callback(self, func):
        self._logger.debug("set_initialize_callback: %s" % (str(func)))
        self._initialize_callback = func

    def set_terminate_callback(self, func):
        self._logger.debug("set_terminate_callback: %s" % (str(func)))
        self._terminate_callback = func

    def set_configuration_ref(self, configuration):
        self._configuration = configuration
        self.__mountdir = self._configuration.get_mountdir()

    def is_initialized(self):
        return self._is_initialized

    def initialize(self):
        """Test existence of path after initialization.
        
        Get the list of dir to mount and mount them. If the keep alive tag is set , it creates a Thread that will keep the mounted dir alive.
        @param keepAlive: Optional int that is used to determine the loop time (number of seconds) to keep the mount pint alive
        
        @todo: For later releases: The distinction between local and remote
               sites must be improved!               
        """
        self._logger.info(_("Initializing FUSE File Access Manager."))
        if self._is_initialized is True:
            raise AssertionError("FUSE File Access Manager is initialized.")

        self._eff_path = None
        self.__check_mountdir()
        self._mount_config_destination()
        self._is_initialized = True

    def __check_mountdir(self):
        """check if the mount dir is valid
        :todo: Do not create dir here? Use makedirs?
        """
        if not os.path.exists(self.__mountdir) :
            os.mkdir(self.__mountdir)
        else:
            if not os.path.isdir(self.__mountdir) :
                raise SBException("The mount base dir should be a directory")

    def terminate(self):
        """
        Unmount all mounted directories.
        """
        self._logger.info(_("Terminating FUSE File Access Manager."))
        error = None
#        print "Mount dirs: %s" % str(self.__mountedDirs)

        if self._is_initialized is True:
            try:
                plugin_manager = PluginManager()
                for src, dir in self.__mountedDirs.iteritems() :
                    self._logger.debug("Mounted dirs - %s - %s" % (src, dir))
                    if src is not os.sep :
                        _umounted = False
                        for p_name, p_class in plugin_manager.getPlugins().iteritems():
                            #we got the plugin
                            self._logger.debug("Trying `%s` plugin to match `%s`" % (p_name, src))
                            plugin = p_class()
                            if plugin.match_scheme(src):
                                self._logger.debug("Unmounting with `%s` plugin" % p_name)
                                plugin.umount(dir)
                                _umounted = True
                                self._remove_mountdir(dir)
                                break
                        if not _umounted:
                            self._logger.warning("Unable to terminate FUSE `%s`" % dir)
            except SBException, error:
                self._logger.error("Error in terminate. Overwriting previous errors.")
                if self._initialize_callback is None:
                    raise

        else:
            self._logger.warning("FUSE File Access Manager is not initialized. Nothing to do.")

        self._eff_path = None

        if self._terminate_callback is not None:
            self._logger.debug("Calling additional callback in gio_fam: %s" % self._terminate_callback)
            self._terminate_callback(error)

        self._is_initialized = False
#        print "End of fuse_fam.terminate"

        self._is_initialized = False

    def __mount_uri(self, uri):
        """Mounts an arbitrary uri and returns effective path. It uses the __mountdir param to know
        where to mount. It also fills the __mountedDirs dict. The configuration is not modified.
        
        @return: effective path of given uri or None if local path was given
        """
        self.__check_mountdir()

        plugin_manager = PluginManager()
        for p_name, p_class in plugin_manager.getPlugins().iteritems():
            try :
                #we got the plugin
                plugin = p_class()
                if plugin.match_scheme_full(uri):
                    self._logger.debug("Processing with plugin '%s' to mount '%s'" % (p_name, uri))
                    rsource, mpoint, pathinside = plugin.mount(uri, self.__mountdir)

                    if rsource != os.sep:   # remote
                        self.__mountedDirs[rsource] = mpoint
#                        print "Mount dirs: %s" % str(self.__mountedDirs)
                        return os.sep.join([mpoint, pathinside])
                    else:
                        # The plugin used was localFuseFAM
                        return None

            except Exception, error:
                _msg = "Unable to mount target destination `%s` using plugin `%s`: %s" % (uri, p_name, error)
                self._logger.error(_msg)
                raise exceptions.FuseFAMException(_msg)

        _msg = "Unable to mount `%s`: schema not supported by plugins." % uri
        self._logger.error(_msg)
        raise exceptions.FuseFAMException(_msg)

    def _mount_config_destination(self):
        """Mounts the destination specified in configuration and modifies the target value
        in configuration accordingly.
        """
        error = None
        try:
            _dest = self.get_destination()
            _eff_path = self.__mount_uri(_dest)
            if _eff_path is None:
                self._eff_path = _dest
            else:
                self._eff_path = _eff_path

            if (self._eff_path is None) or (not self._eff_path.startswith(local_file_utils.PATHSEP)):
                raise exceptions.FileAccessException("Unable to mount target")
            if not local_file_utils.path_exists(self._eff_path):
                raise exceptions.FileAccessException("Unable to mount target: Path does not exist")

        except SBException, error:
            self._logger.error("Error in mount callback function. Overwriting previous errors.")
            if self._initialize_callback is None:
                raise

        if self._initialize_callback is not None:
            self._logger.debug("Calling additional callback in fuse_fam._mount_cb: %s" % self._initialize_callback)
            self._initialize_callback(error)

        if error is None:
            self._is_initialized = True
#        print "End of fuse_fam._mount_config_destination"


    def _remove_mountdir(self, mountpoint):
        """Removes previously created mountpoint in secure manner.
        """
        if os.path.exists(mountpoint):
            if os.path.ismount(mountpoint):
                self._logger.warning("Unable to remove mountpoint that is busy. Unmount before removing.")
            else:
                _listdir = os.listdir(mountpoint)
                if len(_listdir) == 0:
                    try:
                        os.rmdir(mountpoint)
                        self._logger.debug("Mountpoint `%s` successfully removed." % mountpoint)
                    except (IOError, OSError), error:
                        self._logger.error("Unable to remove mountpoint `%s`: %s" % (mountpoint, error))
                else:
                    self._logger.warning("Unable to remove mountpoint: directory is not empty.")
        else:
            self._logger.warning("Unable to remove mountpoint `%s`: does not exist" % mountpoint)

#    def __keepAlive(self):
#        """
#        Launch a command on the mounted dirs to keep the mount alive
#        """
#        pass

#    def testFusePlugins(self, remotedir):
#        """The given remote directory is applied to any found plugins to
#        decide whether one of the plugins is able to handle this (remote)
#        protocol. The tests cover:
#        * checking of the adress scheme and the validity of the adress
#        * mounting of the remote site
#        * write and read access on the remote site.
#        
#        If no plugin is able to handle the given path, an exception is
#        raised.
#        
#        @todo: Customize the raised exception to provided more informations!
#        """
#        if remotedir.startswith(os.sep) :
#            raise SBException("Nothing to do for localpath '%s'." % remotedir)
#
##TODO: inconsistent path handling!
#        # set the defaults 
#        if os.getuid() == 0 :
#            mountdir = "/mnt/sbackup/"
#        else :
#            mountdir = self._configFileHandler.get_user_datadir() + "mountdir"
#
#        # check if the mount dir is valid
#        if not os.path.exists(mountdir) :
#            os.mkdir(mountdir)
#
#        plugin_manager = PluginManager()
#        _plugins = plugin_manager.getPlugins()
#        _iterator = _plugins.iteritems()
#        for p_name, p_class in _iterator:
#            #we got the plugin
#            self._logger.debug("Testing of plugin '%s'" % str(p_name))
#            plugin = p_class()
#            if plugin.match_scheme_full(remotedir):
#                self._logger.debug("Processing with plugin '%s' to mount '%s'" % (p_name, remotedir))
#                try:
#                    rsource, mpoint, pathinside = plugin.mount(remotedir, mountdir)
#                    self._logger.debug("Mount Succeeded !")
#
#                    self._logger.debug("Testing Writability")
#                    test = "testFuseFam"
#                    testfile = os.path.join(mpoint, pathinside, test)
#                    os.mkdir(testfile)
#                    os.rmdir(testfile)
#
#                    plugin.umount(mpoint)
#                    self._remove_mountdir(mpoint)
#
#                except Exception, error:
#                    raise SBException("Test failed with following output:\n\n%s " % error)
#
#                return True
#
#        raise SBException("No plugin could deal with that schema '%s'" % remotedir)

    def get_eff_path(self):
        _effpath = self._eff_path

        assert _effpath != ""
        assert _effpath is not None
        assert _effpath.startswith(local_file_utils.PATHSEP)

        return _effpath

    def get_eff_fullpath(self, *args):
        _base = local_file_utils.normpath(*args)
        _eff_path = self.get_eff_path()
        self._logger.debug("Effective path: `%s`" % _eff_path)
        self._logger.debug("Base path: `%s`" % _base)
        _res = None
        if _eff_path is not None:
            _res = local_file_utils.normpath(_eff_path, _base)
        return _res

    def get_snapshot_path(self, snpname):
        _base = self.query_mount_uri()
        _snppath = local_file_utils.normpath(_base, snpname)
        return _snppath

    def query_dest_fs_info(self):
        (_size, _free) = local_file_utils.query_fs_info(self._eff_path)
        return (_size, _free)

    def query_dest_display_name(self):
        return self._dest.query_display_name()

    def query_mount_uri(self):
        return self.get_eff_path()

    def is_local(self):
        _loc = self._dest.is_local()
        return _loc

    def get_supports_publish(self):
        _res = True
        return _res

    def get_use_iopipe(self):
        _res = False
        return _res

    def dest_path_exists(self):
        """The effective path denotes the local mountpoint of the actual remote or local target.
        It is required in order to give it to TAR as parameter (tar does not support gio).
        It is checked using GIO and native access functions.
        """
        _path = self.query_mount_uri()
        assert _path != ""
        assert _path is not None
        assert _path.startswith(local_file_utils.PATHSEP)
        _res_gio = local_file_utils.path_exists(_path)
        return _res_gio

    def test_destination(self):
        _effpath = self._eff_path

        assert _effpath != ""
        assert _effpath is not None
        assert _effpath.startswith(local_file_utils.PATHSEP)

        dname = "%s-%s-%s.tmp" % ("sbackup-dir", time.time(), uuid.uuid4())
        tfilen = "%s-%s-%s.tmp" % ("sbackup", time.time(), uuid.uuid4())

        local_file_utils.test_path(_effpath, dname, tfilen)



#   Simple Backup - support of DBus functionality
#
#   Copyright (c)2008-2010,2013: Jean-Peer Lorenz <peer.loz@gmx.net>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
"""
:mod:`dbus_support` --- support of DBus functionality
====================================================================

.. module:: dbus_support
   :synopsis: Provides support of DBus functionality
.. moduleauthor:: Jean-Peer Lorenz <peer.loz@gmx.net>

"""

import os
import time
import types
import dbus

from sbackup import dbus_service

from sbackup.util import log
from sbackup.util import notifier
from sbackup.util import structs
from sbackup.util import constants
from sbackup.util import system
from sbackup.util import exceptions
from sbackup.util import get_resource_file


#DBUS_CONNECT_TIMEOUT_TOTAL_SECONDS = 15 # for given number of trials
#DBUS_CONNECT_MAX_TRIALS = 3
DBUS_COMMUNICATE_PAUSE_SECONDS = 0.5  # seconds

KWARG_DBUS_INTERFACE = "dbus_interface"


class _DBusConnection(object):
    """Abstract base class that provides functionality for sending signals
    and calling methods over the dbus.    
    """
    def __init__(self, name):
        """Default constructor.
        
        :param logger: Instance of logger to be used.
        
        """
        self._logger = log.LogFactory.getLogger()
#TODO: maybe it is better to retrieve the logger on demand?

        self._name = name
        self._id = ""
        self._system_bus = None
        self._backup_obj = None
        self._connection_obj = None
        self._connected = False
        self._dbus_id = None

        self.__clear()

    def __clear(self):
        self._id = ""
        self._system_bus = None
        self._backup_obj = None
        self._connection_obj = None
        self._connected = False
        self._dbus_id = None

    def get_is_connected(self):
        return self._connected

#    def __do_connect(self, service, path, silent = False):
#        """Private helper method for connecting to service (i.e. importing the
#        remote object via DBus).
#        """
#        remote_obj = None
#        timeout = DBUS_CONNECT_TIMEOUT_TOTAL_SECONDS
#        max_trials = DBUS_CONNECT_MAX_TRIALS
#        dur = timeout / max_trials
#
#        trials = 0          # trials performed
#        connecting = True
#        while connecting:
#            try:
#                trials += 1
#                self._logger.debug("Trying to connect to `%s` (trial no. %s)" % (service,
#                                                                    trials))
#                remote_obj = self._system_bus.get_object(service, path)
#                connecting = False
#                if not silent:
#                    self._logger.info("successfully connected to `%s` provided by `%s`" % (path, service))
#            except dbus.DBusException, error:
#                if not silent:
#                    self._logger.error("Error while getting service:\n%s" % (str(error)))
#                if trials == max_trials:
#                    self._logger.debug("Number of max. trials reached - timeout!")
#                    connecting = False
#                    remote_obj = None
#                else:
#                    self._logger.debug("Waiting %s sec. before re-connecting" % dur)
#                    time.sleep(dur)
#        return remote_obj

    def __do_connect(self, service, path, silent = False):
        """Private helper method for connecting to service (i.e. importing the
        remote object via DBus).
        """
        remote_obj = None
        try:
            self._logger.debug("Trying to connect to `%s`" % service)
            remote_obj = self._system_bus.get_object(service, path)
            if not silent:
                self._logger.info("successfully connected to `%s` provided by `%s`" % (path, service))
        except dbus.DBusException, error:
            if not silent:
                self._logger.error("Error while getting service:\n%s" % (str(error)))
            remote_obj = None
        return remote_obj

    def connect(self, silent = True):
        """Connects passively to DBus service and registers this connection at the service.
        The service id is stored for later checks of the validity of the connection.
        
        Remote objects are set to None in case of failure. 
        """
        self.test_validity()
        if self._connected is True:
            if not silent:
                self._logger.info("Connection is already established.")
        else:
            _failure = False
            try:
                self._system_bus = dbus.SystemBus()
                self._backup_obj = self.__do_connect(constants.DBUS_SERVICE,
                                                     constants.DBUS_OBJ_PATH,
                                                     silent)
                self._connection_obj = self.__do_connect(constants.DBUS_SERVICE,
                                                     constants.DBUS_CONNECTION_OBJ_PATH,
                                                     silent)

                if (self._connection_obj is not None) and (self._backup_obj is not None):
                    pid = os.getpid()
                    self._id = self._connection_obj.register_connection(self._name, str(pid))
                    self._dbus_id = self._connection_obj.get_id()
                    self._logger.debug("Registered with id: %s" % self._id)
                    self._connected = True
                else:
                    _failure = True
                    error = "Unable to get remote object"

            except dbus.DBusException, error:
                _failure = True

            if _failure is True:
                self.__clear()
                if not silent:
                    self._logger.error("Error while connecting to DBus service: %s" % error)
            else:
                self.test_validity()
#                if self._connected:
#                    if not silent:
#                        self._logger.info("Successfully connected to Dbus service with id: %s" % self._dbus_id)
#                else:
#                    self._logger.warning("Unable to connect to Dbus service")

    def test_validity(self):
        """Tests whether connection is valid, i.e. exists and is the same connection as originally
        connected to. If connection is `invalid` everything is cleared and external callers (that use
        this instance) are required to call `connect` again. Moreover, it is required to re-connect any
        signal handlers. (This method does not return the result of the test due to CQS. Call
        `get_is_connected` instead.)
        """
        valid = False
        failure = False
        dbus_id = None

        if self._connection_obj is not None:
            try:
                dbus_id = self._connection_obj.get_id()
            except dbus.DBusException, error:
                self._logger.error("Unable to get service while testing availibility: %s" % (str(error)))
                dbus_id = None
                failure = True

        if self._backup_obj is not None:
            try:
                self._backup_obj.get_backup_pid()
            except dbus.DBusException, error:
                self._logger.error("Unable to get service while testing availibility: %s" % (str(error)))
                failure = True

        if (failure is False) and (dbus_id is not None)\
           and (self._id is not None) and (dbus_id == self._dbus_id):
            valid = True
        else:
            valid = False

        if valid is False:
            self.__clear()

    def quit(self):
        res = True
        if self._connected is True:
            if self._id != "":
                if self._connection_obj is not None:
                    try:
                        res = bool(self._connection_obj.unregister_connection(self._id))
                    except dbus.DBusException, error:
                        self._logger.warning("Unable to unregister connection: %s" % (str(error)))
                        res = False

                    if res is True:
                        self._logger.debug("Connection was successfully unregistered.")
                    else:
                        self._logger.warning("Unable to unregister connection (failed).")
#            else:
#                print "WARN: Unable to unregister connection (no client id)."
# avoid warning when calling this method as singleton
        self.__clear()
        return res


class DBusProviderFacade(_DBusConnection):
    """This class provides functionality for sending signals
    and calling methods over the dbus.
     
    The Dbus connection is only created on demand. In the case the user
    don't want to use it, no connection is created.
    
    The Provider is the only class that is able to send an Exit signal over the
    dbus. 
    
    :todo: Singleton!
    """
    __metaclass__ = structs.Singleton

    def __init__(self, name):
        """Default constructor.
        
        """
        _DBusConnection.__init__(self, name)

    def _launch_dbusservice(self):
        """Launches the DBus service and establishes a placeholder
        connection in order to keep the service alive as long as this
        application is running. Call `finalize` to close the
        connection properly when terminating the application.
        """
        dbus_launcher = get_resource_file(constants.DBUSSERVICE_FILE)
        system.exec_command_async(args = [dbus_launcher, "start"])
        time.sleep(DBUS_COMMUNICATE_PAUSE_SECONDS)

    def ensure_connectivity(self):
        """Tests whether a connection is established and if not tries to connect.
        If necessary, the service is being launched.  
        """
        self.test_validity()
        if self.get_is_connected() is False:
            self.connect()
            if self.get_is_connected() is False:
                raise exceptions.DBusException("Unable to launch DBus service")

        assert self._connected is True
        assert self._backup_obj is not None
        assert self._connection_obj is not None


    def connect(self, silent = False):
        """Connects actively to DBus service and registers this connection at the service.
        If necessary, the service is being launched. The service id is stored for later
        checks of the validity of the connection.        
        """
        if dbus_service.is_running() is False:
            self._launch_dbusservice()
        _DBusConnection.connect(self)

    def __call_remote(self, func, *args, **kwargs):
        """Helper method for setter and signal emitter.
        """
        if not kwargs.has_key(KWARG_DBUS_INTERFACE):
            kwargs[KWARG_DBUS_INTERFACE] = constants.DBUS_INTERFACE
        res = False
        # we can not re-connect from this method since this would make given callable func invalid
        if self.get_is_connected() is True:
            try:
                res = func(*args, **kwargs)
            except dbus.DBusException, error:
                self._logger.error("Error while DBus operation in `__call_remote`: %s\n(%s(%s, %s))" % \
                                   (error, str(func), str(args), str(kwargs)))
                res = False
        res = bool(res)
        return res

    def emit_event_signal(self, event, urgency):
        """Used for sending a generic event over the signal dbus.
        This includes informations and warnings.
        
        :param event: the actually processed event
        :param urgency: how urgent the message is
        """
        if not isinstance(event, types.StringTypes):
            raise TypeError("Parameter `event` of string type expected. Got %s instead." % str(type(event)))
        if not isinstance(urgency, types.StringTypes):
            raise TypeError("Parameter `urgency` of string type expected. Got %s instead." % str(type(urgency)))
        self.ensure_connectivity()
        res = self.__call_remote(self._backup_obj.emit_event_signal, event, urgency)
        assert isinstance(res, types.BooleanType)
        return res

    def emit_error_signal(self, error):
        """Used for sending an error signal over the signal dbus.
        
        :param profile: name of the current profile
        :param error: error message to be passed
        
        """
        if not isinstance(error, types.StringTypes):
            raise TypeError("Parameter of string type expected. Got %s instead." % str(type(error)))
        self.ensure_connectivity()
        res = self.__call_remote(self._backup_obj.emit_error_signal, error)
        assert isinstance(res, types.BooleanType)
        return res

    def emit_targetnotfound_signal(self):
        self.ensure_connectivity()
        res = self.__call_remote(self._backup_obj.emit_targetnotfound_signal)
        assert isinstance(res, types.BooleanType)
        return res

    def set_backup_pid(self, pid):
        if not isinstance(pid, types.IntType):
            raise TypeError("Parameter of integer type expected. Got %s instead." % str(type(pid)))
        self.ensure_connectivity()
        res = self.__call_remote(self._backup_obj.set_backup_pid, pid)
        assert isinstance(res, types.BooleanType)
        return res

    def set_target(self, target):
        if not isinstance(target, types.StringTypes):
            raise TypeError("Parameter of string type expected. Got %s instead." % str(type(target)))
        self.ensure_connectivity()
        res = self.__call_remote(self._backup_obj.set_target, target)
        assert isinstance(res, types.BooleanType)
        return res

    def set_profilename(self, profilename):
        if not isinstance(profilename, types.StringTypes):
            raise TypeError("Parameter of string type expected. Got %s instead." % str(type(profilename)))
        self.ensure_connectivity()
        res = self.__call_remote(self._backup_obj.set_profilename, profilename)
        assert isinstance(res, types.BooleanType)
        return res

    def set_space_required(self, space):
        if not isinstance(space, (types.IntType, types.LongType)):
            raise TypeError("Parameter of string type expected. Got %s instead." % str(type(space)))
        space = long(space)
        self.ensure_connectivity()
        res = self.__call_remote(self._backup_obj.set_space_required, space)
        assert isinstance(res, types.BooleanType)
        return res

    def get_retry_target_check(self):
        """This method is an exception from intended strict separation of setter/getter
        methods into Provider/Client objects.
        
        Keep this method passive!
        
        :todo: Improve design of dbus service related code.
        """
        retval = constants.RETRY_UNKNOWN
        self.test_validity()
        if self.get_is_connected() is True:
            try:
                retval = self._backup_obj.get_retry_target_check(dbus_interface = constants.DBUS_INTERFACE)
            except dbus.DBusException, error:
                self._logger.error("Error while dbus operation in `get_retry_target_check`: %s" % error)
                retval = constants.RETRY_UNKNOWN
        return retval

    def exit(self):
        """
        """
        self.ensure_connectivity()
        self.__call_remote(self._connection_obj.emit_exit_signal)
        res = self.quit()
        return res


class DBusClientFacade(_DBusConnection):
    """This class provides functionality for sending signals
    and calling methods over the dbus.
    
    The sender needs a dbus connection.
    
    The Dbus connection is only created on demand. In the case the user
    don't want to use it, no connection is created.
    
    """
    def __init__(self, name):
        """Default constructor.        
        """
        _DBusConnection.__init__(self, name)

    def connect_to_event_signal(self, handler):
#TODO: raise an exception here when not connected to remote object?
        self.test_validity()
        if self.get_is_connected() is True:
            self._backup_obj.connect_to_signal('event_signal', handler,
                                               dbus_interface = constants.DBUS_INTERFACE)

    def connect_to_error_signal(self, handler):
        self.test_validity()
        if self.get_is_connected() is True:
            self._backup_obj.connect_to_signal('error_signal', handler,
                                               dbus_interface = constants.DBUS_INTERFACE)

    def connect_to_progress_signal(self, handler):
        self.test_validity()
        if self.get_is_connected() is True:
            self._backup_obj.connect_to_signal('progress_signal', handler,
                                               dbus_interface = constants.DBUS_INTERFACE)

    def connect_to_targetnotfound_signal(self, handler):
        self.test_validity()
        if self.get_is_connected() is True:
            self._backup_obj.connect_to_signal('targetnotfound_signal', handler,
                                               dbus_interface = constants.DBUS_INTERFACE)

    def connect_to_alreadyrunning_signal(self, handler):
        self.test_validity()
        if self.get_is_connected() is True:
            self._backup_obj.connect_to_signal('alreadyrunning_signal', handler,
                                               dbus_interface = constants.DBUS_INTERFACE)

    def connect_to_exit_signal(self, handler):
        signal = "exit_signal"
        self.test_validity()
        if self.get_is_connected() is True:
            self._connection_obj.connect_to_signal(signal, handler,
                                                   dbus_interface = constants.DBUS_INTERFACE)

    def get_backup_pid(self):
        retval = constants.PID_UNKNOWN
        self.test_validity()
        if self.get_is_connected() is True:
            try:
                retval = self._backup_obj.get_backup_pid(dbus_interface = constants.DBUS_INTERFACE)
            except dbus.DBusException, error:
                self._logger.error("Error while dbus operation in `get_backup_pid`: %s" % error)
                retval = constants.PID_UNKNOWN
        return retval

    def get_target(self):
        retval = constants.TARGET_UNKNOWN
        self.test_validity()
        if self.get_is_connected() is True:
            try:
                retval = self._backup_obj.get_target(dbus_interface = constants.DBUS_INTERFACE)
            except dbus.DBusException, error:
                self._logger.error("Error while dbus operation in `get_target`: %s" % error)
                retval = constants.TARGET_UNKNOWN
        return retval

    def get_profilename(self):
        retval = constants.PROFILE_UNKNOWN
        self.test_validity()
        if self.get_is_connected() is True:
            try:
                retval = self._backup_obj.get_profilename(dbus_interface = constants.DBUS_INTERFACE)
            except dbus.DBusException, error:
                self._logger.error("Error while dbus operation in `get_profilename`: %s" % error)
                retval = constants.PROFILE_UNKNOWN
        return retval

    def get_space_required(self):
        retval = constants.SPACE_REQUIRED_UNKNOWN
        self.test_validity()
        if self.get_is_connected() is True:
            try:
                retval = self._backup_obj.get_space_required(dbus_interface = constants.DBUS_INTERFACE)
            except dbus.DBusException, error:
                self._logger.error("Error while dbus operation in `get_space_required`: %s" % error)
                retval = constants.SPACE_REQUIRED_UNKNOWN
        return retval

    def set_retry_target_check(self, retry):
        """This method is an exception from intended strict separation of setter/getter
        methods into Provider/Client objects.
        :todo: Improve design of dbus service related code.
        """
        res = False
        self.test_validity()
        if self.get_is_connected() is True:
            try:
                res = self._backup_obj.set_retry_target_check(retry,
                                                              dbus_interface = constants.DBUS_INTERFACE)
            except dbus.DBusException, error:
                self._logger.error("Error while setting value for `retry_target_check`: %s" % error)
                res = False
        else:
            self._logger.warning("Unable to set value for `retry_target_check`: No connection available.")
        res = bool(res)
        return res

    def emit_alreadyrunning_signal(self):
        res = False
        self.test_validity()
        if self.get_is_connected() is True:
            try:
                res = self._backup_obj.emit_alreadyrunning_signal(dbus_interface = constants.DBUS_INTERFACE)
            except dbus.DBusException, error:
                self._logger.error("Error while emitting `alreadyrunning_signal`: %s" % error)
                res = False
        else:
            self._logger.warning("Unable to emit `alreadyrunning_signal`: No connection available.")
        res = bool(res)
        return res



class DBusNotifier(notifier.Observer):
    """Sends notifications as signals over the DBus.
    It uses an instance of DBusConnection for signaling.
    
    """
    def __init__(self):
        notifier.Observer.__init__(self)

        self.__state = None
        self.__urgency = None
        self.__profilename = None
        self.__recent_error = None
        self.__target = None
        self.__space_required = None

        self.__logger = log.LogFactory.getLogger()
        self.__dbus = None

    def initialize(self):
        self.__setup_dbus()

    def publish_exit(self):
        """
        :todo: Rename into something better (e.g. publish_exit?)
        """
        try:
            self.__dbus.exit()
        except exceptions.DBusException:
            self.__logger.warning("Unable to publish exit over D-Bus")

    def __setup_dbus(self):
        self.__dbus = DBusProviderFacade(constants.DBUS_NOTIFIER_NAME)
        self.__dbus.connect()

    def update(self, subject):
        """Interface method for observer objects that is called by the
        observed subject. In the case of the `DBusNotifier` were
        notifications send over the bus.
         
        """
        self.__logger.debug("Observer update")
        self.__state = subject.get_state()
        self.__urgency = subject.get_urgency()
        self.__profilename = subject.get_profilename()
        self.__recent_error = subject.get_recent_error()
        self.__target = subject.get_target()
        self.__space_required = subject.get_space_required()

        self.__attempt_notify()

    def __attempt_notify(self):
        """Private helper method that actually tries to notify over
        the DBus about several events. This method decides what
        signals are send over the bus.
        
        :return: the value returned by the signal (usually True)
        
        :todo: Remove urgency!
        
        """
        state = self.__state
        urgency = self.__urgency
        self.__logger.debug("state: `%s` - urgency: `%s`" % (state, urgency))

        ret_val = False
        if self.__dbus is not None:
            try:
                # update all properties
                ret_val = self.__dbus.set_target(self.__target)
                ret_val = self.__dbus.set_profilename(self.__profilename)
                ret_val = self.__dbus.set_space_required(self.__space_required)

                if state in ('start',
                             'finish',
                             'prepare',
                             'commit',
                             'backup-canceled'):
                    ret_val = self.__dbus.emit_event_signal(state, urgency)

                elif state == 'error':
                    ret_val = self.__dbus.emit_error_signal(str(self.__recent_error))

                elif state == 'target-not-found':
                    ret_val = self.__dbus.emit_targetnotfound_signal()

                else:
                    raise ValueError("STATE UNSUPPORTED (%s)" % state)
            except exceptions.DBusException:
                self.__logger.warning("Unable to notifiy over D-Bus")
                ret_val = False

        return ret_val


def get_session_name():
    # a full DE is supposed
    session = ""
    gnome_found = False
    kde_found = False
    try:
        bus = dbus.SessionBus()
        bus.get_object('org.kde.ksmserver', '/KSMServer')
        session = "kde"
        kde_found = True
    except dbus.exceptions.DBusException, error:
        print "Unable to get KDE Session Manager: %s" % error
        kde_found = False

    try:
        bus = dbus.SessionBus()
        bus.get_object('org.gnome.SessionManager',
                                       '/org/gnome/SessionManager')
        session = "gnome"
        gnome_found = True
    except dbus.exceptions.DBusException, error:
        print "Unable to get Gnome Session Manager: %s" % error
        gnome_found = False

    if gnome_found and kde_found:
        raise AssertionError("Unable to get non-ambiguous desktop session: Gnome *and* KDE found")

    return session



#   Simple Backup - interface definitions
#
#   Copyright (c)2010: Jean-Peer Lorenz <peer.loz@gmx.net>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#


def _get_notimplemented_msg(klass, method):
    _res = "Method `%s` is abstract in class `%s` but is not overridden" % (method, klass)
    return _res


class ITargetHandler(object):

    def __init__(self):
        pass

    def initialize(self):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "initialize"))

    def is_initialized(self):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "is_initialized"))

    def terminate(self):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "terminate"))

    def dest_path_exists(self):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "dest_path_exists"))

    def query_dest_display_name(self):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "query_dest_display_name"))

    def query_mount_uri(self):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "query_mount_uri"))

    def query_dest_fs_info(self):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "query_dest_fs_info"))

    def test_destination(self):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "test_destination"))

    def set_initialize_callback(self, func):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "set_initialize_callback"))

    def set_terminate_callback(self, func):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "set_terminate_callback"))

    def set_destination(self, path):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "set_destination"))

    def get_destination(self):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "get_destination"))

    def set_configuration_ref(self, configuration):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "set_configuration_ref"))

    def get_eff_path(self):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "get_eff_path"))

    def get_eff_fullpath(self, *args):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "get_eff_fullpath"))

    def get_snapshot_path(self, snpname):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "get_snapshot_path"))

    def is_local(self):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "is_local"))

    def get_supports_publish(self):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "get_supports_publish"))

    def get_use_iopipe(self):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "get_use_iopipe"))

    def set_use_mainloop(self, use):
        raise NotImplementedError(_get_notimplemented_msg("ITargetHandler", "set_use_mainloop"))


class IOperations(object):

    def __init__(self):
        pass

    @classmethod
    def path_exists(cls, path):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "path_exists"))

    @classmethod
    def delete(cls, uri):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "delete"))

    @classmethod
    def force_delete(cls, path):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "force_delete"))

    @classmethod
    def makedir(cls, path):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "makedir"))

    @classmethod
    def makedirs(cls, path):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "makedirs"))

    @classmethod
    def normpath(cls, *args):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "normpath"))

    @classmethod
    def openfile_for_write(cls, path):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "openfile_for_write"))

    @classmethod
    def openfile_for_read(cls, path):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "openfile_for_read"))

    @classmethod
    def openfile_for_append(cls, path):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "openfile_for_append"))

    @classmethod
    def get_dirname(cls, path):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "get_dirname"))

    @classmethod
    def get_basename(cls, path):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "get_basename"))

    @classmethod
    def pickleload(cls, path):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "pickleload"))

    @classmethod
    def pickledump(cls, datas, path):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "pickledump"))

    @classmethod
    def rename(cls, src, dst):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "rename"))

    @classmethod
    def readfile(cls, uri):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "readfile"))

    @classmethod
    def writetofile(cls, path, content):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "writetofile"))

    @classmethod
    def copyfile(cls, src, dest):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "copyfile"))

    @classmethod
    def listdir(cls, target) :
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "listdir"))

    @classmethod
    def listdir_fullpath(cls, target) :
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "listdir_fullpath"))

    @classmethod
    def force_move(cls, src, dst):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "force_move"))

    @classmethod
    def is_link(cls, path):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "is_link"))

    @classmethod
    def is_dir(cls, path):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "is_dir"))

    @classmethod
    def close_stream(cls, file_desc):
        raise NotImplementedError(_get_notimplemented_msg("IOperations", "close_stream"))



#   Simple Backup - fuse plugins framework (package) 
#
#   Copyright (c)2009-2010: Jean-Peer Lorenz <peer.loz@gmx.net>
#   Copyright (c)2007-2008: Ouattara Oumar Aziz <wattazoum@gmail.com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

from gettext import gettext as _

import inspect
import os
import sys
import glob
import tempfile
import subprocess


import sbackup
from sbackup.util import local_file_utils
from sbackup.util import log
from sbackup.util import exceptions


class pluginFAM(object):
    """
    The fuseFAM plugin interface
    @author: Oumar Aziz Ouattara <wattazoum@gmail.com>
    """

    def __init__(self):
        self.logger = log.LogFactory.getLogger()

    def match_scheme(self, remoteSource):
        raise exceptions.SBException("'match_scheme' Not implemented for this plugin")

    def match_scheme_full(self, remoteSource):
        """
        Try to match the scheme of the remoteSource.
        @param remoteSource: The remote path
        @return: True if the scheme matches the one for this 
        @rtype: boolean
        """
        raise exceptions.SBException("'match_scheme_full' Not implemented for this plugin")

    def mount(self, source, mountbase):
        """
        Mount the source intor the mountbase dir . This method should create a mount point to mount the source. 
        The name of the mount point should be very expressive so that we avoid collision with other mount points
        This method will return a tuple (baseRemoteSource, mountpoint, pathinside) where
        - baseRemoteSource is the substring that represent the mount source (usually at the start of the source). The match_scheme_full method should be able to match it
        - mountpoint is the mount point of this baseRemoteSource.
        - pathinside is the path inside the remote source 
        [Use case]
        The mount function is called once with "sch://server/path/to/dir" and the plugin mounts "sch://server" to "/mnt/sch_server". 
        Then the return value would be ("sch://server","/mnt/sch_server","path/to/dir")
        @param source: The remote path
        @param mountbase: The mount points base dir
        @return: The tuple (baseRemoteSource, mountpoint, pathinside)
        @rtype: tuple
        """
        raise exceptions.SBException("'mount' Not implemented for this plugin")

    def umount(self, mounteddir):
        """
        Default behaviour is to unmount with fuse
        """
        if os.path.ismount(mounteddir):
            self.logger.debug("Unmounting `%s`" % mounteddir)
            # Create output and error log file
            outptr, outFile = tempfile.mkstemp(prefix = "fuseUmount_output_")
            errptr, errFile = tempfile.mkstemp(prefix = "fuseUmount_error_")

            # Call the subprocess using convenience method using lazy umount
            retval = subprocess.call(["fusermount", "-u", "-z", mounteddir], 0, None, None, outptr, errptr)

            # Close log handles
            os.close(errptr)
            os.close(outptr)
            outStr, errStr = local_file_utils.readfile(outFile), local_file_utils.readfile(errFile)
            local_file_utils.delete(outFile)
            local_file_utils.delete(errFile)

            self.logger.debug("fusermount output:\n%s\n%s" % (outStr, errStr))

            if retval != 0 :
                raise exceptions.SBException("Unable to unmount `%s`: %s" % (mounteddir, errStr))
            else:
                self.logger.info("Successfully unmounted: `%s`" % mounteddir)
        else:
            self.logger.warning("Unable to unmount `%s`: not mounted" % mounteddir)

    def checkifmounted (self, source, mountbase):
        """
        Should check if the source is mounted.
        Note : you should use os.path.ismount(path) method for that, after determining the name of the mount point.
        @return: True if it is, False if not
        """
        raise exceptions.SBException("'Check if mounted' Not implemented for this plugin")

    def getdoc(self):
        """
        This method should give a little documentation about the schema used for this plugin.
        @return: The schema doc (eg. return 'example : sch://user:password@server/dir')
        """
        raise exceptions.SBException("Help not implemented for this plugin")


class PluginManager(object):
    """
    """
    def __init__(self):
        self.logger = log.LogFactory.getLogger()

        # This should be a dictionary of plugins
        self.__pluginList = None

    def getPlugins(self):
        """Searches for plugins in the plugin directory and loads them.
        
        @return : The plugins dictionary list {'name':class}.
        @note: Look at FuseFAM to know how it's used.
        """

        if self.__pluginList is not None:
            return self.__pluginList

        else:
            self.__pluginList = dict()
            tmp = inspect.getabsfile(inspect.getmodule(self))
            plugins_dir = os.path.dirname(tmp)
            if os.path.isdir(plugins_dir):
                if plugins_dir not in sys.path:
                    sys.path.append(plugins_dir)

                for _file in glob.glob('%s/*FuseFAM.py' % plugins_dir):
                    try:
                        module_filename = os.path.basename(_file)
                        module_name, mod_ext = os.path.splitext(module_filename)  # IGNORE:W0612
                        plugin = __import__(module_name, '', module_filename)
                        for symbol_name in dir(plugin):
                            symbol = getattr(plugin, symbol_name)
                            if inspect.isclass(symbol) and symbol != pluginFAM and \
                                issubclass(symbol, pluginFAM):
                                #symbol.enabled = symbol.name in plugin_names
                                self.__pluginList[symbol_name] = symbol
                    except Exception, error:
                        self.logger.warning(_("Unable to import plugin `%(plugin_name)s`: %(error_cause)s ")\
                                            % { 'plugin_name' : module_name,
                                                'error_cause' : str(error) })
                        continue
            return self.__pluginList
